/**
 * @file  AE3Buffer.cpp
 * @brief Implementierung der Klasse CAE3Buffer.
 */
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "AE3Buffer.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CAE3Buffer::CAE3Buffer(unsigned char ucChannelNum)
{
	// Kanalnummer zuweisen
	m_ucChannelNum = ucChannelNum;

	// Member Initialisieren
	StartDatensatzB = 0;
	EndDatensatzB = 0;
	MaxDatB = 0;
	MaxListB = 0;
	
	// Listenpuffer leeren.
	ClearListBuffer();

	// Datensatzgroesse fuer Zutrittslisten initialisieren.
	m_ucEntranceRecordSize[0] = ENTRANCE_RECORD_SIZE_0; // 0: Zeitzonen (22)
	m_ucEntranceRecordSize[1] = ENTRANCE_RECORD_SIZE_1; // 1: Zeitmodelle (35)
	m_ucEntranceRecordSize[2] = ENTRANCE_RECORD_SIZE_2; // 2: Personalstamm (19)
	m_ucEntranceRecordSize[3] = ENTRANCE_RECORD_SIZE_3; // 3: Feiertage (16)

	// Seriennummer der Geraetedatei auf 0 setzen
	m_fileDevice.SetSerial(0);
}

CAE3Buffer::~CAE3Buffer()
{

}

//////////////////////////////////////////////////////////////////////
// Member
//////////////////////////////////////////////////////////////////////

void CAE3Buffer::ChangeOrderLong(unsigned char *Buf)
{
	unsigned char h;

	h = Buf[0];
	Buf[0] = Buf[3];
	Buf[3] = h;

	h = Buf[1];
	Buf[1] = Buf[2];
	Buf[2] = h;
}

void CAE3Buffer::ULongToBinMSBFirst(unsigned char *Buf,unsigned long Val)
{
	Buf[3] = (unsigned char)(Val&0xFF);
	Val >>= 8;
	Buf[2] = (unsigned char)(Val&0xFF);
	Val >>= 8;
	Buf[1] = (unsigned char)(Val&0xFF);
	Val >>= 8;
	Buf[0] = (unsigned char)(Val&0xFF);
}

void CAE3Buffer::ChangeOrderInt(unsigned char *Buf)
{
	unsigned char h;
	h = Buf[0];
	Buf[0] = Buf[1];
	Buf[1] = h;
}

int CAE3Buffer::MakeListe(int ListNum, int ListCnt, int ListSize, unsigned char *Buf)
{
	unsigned long pDat;
	int i;

	// Startdadresse Listen ist immer FLASH_LIST_START.
	m_ulListStart = FLASH_LIST_START;
	
	// Pointer in Tabelle der Listen-Pointer eintragen
	*(unsigned long *)(&(m_pucListMem[ListNum * sizeof(ULONG)])) = m_ulListSize + m_ulListStart;
	ChangeOrderLong((unsigned char*)&(m_pucListMem[ListNum * sizeof(ULONG)]));

	// Start fuer Listendaten berechnen
	pDat = m_ulListSize + sizeof(ULONG);	// Platz fuer Anzahl Zeilen freihalten

	// Datenueberlauf auffangen
	if ((pDat + ListSize) > sizeof(m_pucListMem)) {
		// Liste loeschen
		*(unsigned long *)(&(m_pucListMem[ListNum * sizeof(ULONG)])) = 0;
		return FALSE;
	}
	
	// Listendaten kopieren
	for (i=0; i < ListSize; i++) {

		m_pucListMem[pDat + i] = Buf[i];
	}

	// Anzahl Listenzeilen eintragen
	*(unsigned long*)(&(m_pucListMem[m_ulListSize])) = ListCnt;
	ChangeOrderLong(&(m_pucListMem[m_ulListSize]));
	
	// Listen-Pointer auf erstes freies Byte im Listenspeicher setzen
	m_ulListSize = pDat + ListSize;

	return TRUE;
}

void CAE3Buffer::ClearListBuffer()
{
	m_ulListSize = MAX_DATENSATZ_BESCHREIBUNG * sizeof(ULONG);

	for(unsigned int i=0; i < m_ulListSize; i++) {

		m_pucListMem[i] = 0;
	}
}

int CAE3Buffer::MakeEntranceList(int ListNum, int ListCnt, int ListSize, unsigned char *Buf)
{
	int i;

	// ListNum ist:
	// 0: Zeitzonen     (22 Byte) zu komprimierten  (6 Byte) max.   16 DS moeglich
	// 1: Zeitmodelle   (35 Byte) zu komprimierten  (3 Byte) max.   32 DS moeglich
	// 2: Personalstamm (19 Byte) zu komprimierten (16 Byte) max. 5120 DS moeglich
	// 3: Feiertage     (16 Byte) zu komprimierten  (6 Byte) max.   64 DS moeglich
	
	// Rohe Listendaten kopieren
	for (i=0; i < ListSize; i++) {

		m_pucEntranceListMem[ListNum][sizeof(ULONG) + i] = Buf[i]; // Platz fuer Anzahl Zeilen freihalten
	}


	// Anzahl Listenzeilen eintragen
	*(unsigned long*)(&(m_pucEntranceListMem[ListNum][0])) = ListCnt;
	ChangeOrderLong(&(m_pucEntranceListMem[ListNum][0]));
	
	// Listenspeichergroesse setzen
	m_ulEntranceListSize[ListNum] = sizeof(ULONG) + ListSize;


	// ##########################################
	// Komprimieren und sortieren der Listendaten
	// ##########################################

	// Liste Zeitzone - Komprimieren
	if (ListNum == 0) {

		structZeitZone *pZeitZone;
		unsigned long i, j;
		unsigned char ucRecordSize = ENTRANCE_RECORD_SIZE_0;
		unsigned char pucRecord[ENTRANCE_RECORD_SIZE_0];

		// Anzahl Listenzeilen eintragen
		*(unsigned long*)(&(m_pucCompressedListMem[ListNum][0])) = ListCnt;
		ChangeOrderLong(&(m_pucCompressedListMem[ListNum][0]));

		m_ulCompressedListSize[ListNum] = sizeof(ULONG);

		// Liste mit -zeitzone- Daten komprimieren
		for (i=0; i < (ULONG)ListCnt; i++) {
			
			for (j=0; j < ucRecordSize; j++) {

				// Datensatz aufbereiten.
				if (m_pucEntranceListMem[ListNum][sizeof(ULONG) + i * ucRecordSize + j]) {
					// Abzug fuer ASCII - Werte
					pucRecord[j] = m_pucEntranceListMem[ListNum][sizeof(ULONG) + i * ucRecordSize + j] - 0x30;
				} else {
					pucRecord[j] = 0x00;
				}
			}

			pZeitZone = (structZeitZone*)&m_pucCompressedListMem[ListNum][sizeof(ULONG) + i * sizeof(structZeitZone)];
			pZeitZone->day = (UCHAR)((1 << pucRecord[0]) | (1 << pucRecord[1]) | (1 << pucRecord[2]) | (1 << pucRecord[3]) | (1 << pucRecord[4]) | (1 << pucRecord[5]) | (1 << pucRecord[6]));
			pZeitZone->start = ((unsigned short)(pucRecord[8] * 10 + pucRecord[9]) << 8) | (pucRecord[11] * 10 + pucRecord[12]);
			ChangeOrderInt((UCHAR*)&pZeitZone->start);
			pZeitZone->ende = ((unsigned short)(pucRecord[14] * 10 + pucRecord[15]) << 8) | (pucRecord[17] * 10 + pucRecord[18]);
			ChangeOrderInt((UCHAR*)&pZeitZone->ende);
			pZeitZone->aktiv = pucRecord[20];

			// Strukturoffsets zu Listengroesse aufrechnen.
			m_ulCompressedListSize[ListNum] += sizeof(structZeitZone);
		}
	}

	// Liste Zeitmodelle - Komprimieren
	if (ListNum == 1) {

		structZeitModell *pZeitModell;
		unsigned long i, j;
		unsigned char ucRecordSize = ENTRANCE_RECORD_SIZE_1;
		unsigned char pucRecord[ENTRANCE_RECORD_SIZE_1];
		unsigned char ucOffs = 0;

		// Anzahl Listenzeilen eintragen
		*(unsigned long*)(&(m_pucCompressedListMem[ListNum][0])) = ListCnt;
		ChangeOrderLong(&(m_pucCompressedListMem[ListNum][0]));

		m_ulCompressedListSize[ListNum] = sizeof(ULONG);

		// Liste mit -zeitzmodelle- Daten komprimieren
		for (i=0; i < (ULONG)ListCnt; i++) {
			
			for (j=0; j < ucRecordSize; j++) {

				// Datensatz aufbereiten.
				if (m_pucEntranceListMem[ListNum][sizeof(ULONG) + i * ucRecordSize + j]) {
					// Abzug fuer ASCII - Werte
					pucRecord[j] = m_pucEntranceListMem[ListNum][sizeof(ULONG) + i * ucRecordSize + j] - 0x30;
				} else {
					pucRecord[j] = 0x00;
				}
			}

			pZeitModell = (structZeitModell*)&m_pucCompressedListMem[ListNum][sizeof(ULONG) + i * sizeof(structZeitModell)];
			pZeitModell->zeitmodell = pucRecord[0] * 10 + pucRecord[1];
			ucOffs = 3;
			pZeitModell->zone = 0;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0001 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0002 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0004 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0008 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0010 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0020 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0040 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0080 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0100 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0200 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0400 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x0800 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x1000 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x2000 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x4000 : 0x0000;
			ucOffs += 2;
			pZeitModell->zone |= (pucRecord[ucOffs]) ? 0x8000 : 0x0000;

			ChangeOrderInt((UCHAR*)&pZeitModell->zone);

			// Strukturoffsets zu Listengroesse aufrechnen.
			m_ulCompressedListSize[ListNum] += sizeof(structZeitModell);
		}
	}

	// Liste Personalstamm - Komprimieren
	if (ListNum == 2) {
		
		// Personalstamm nach Personalnummer sortieren.
		QuicksortEntranceRecord(ListNum, 0, 0, ListCnt - 1);


		structPersonalStamm *pPersonalStamm;
		unsigned long i, j;
		unsigned char ucRecordSize = ENTRANCE_RECORD_SIZE_2;
		unsigned char pucRecord[ENTRANCE_RECORD_SIZE_2];

		// Anzahl Listenzeilen eintragen
		*(unsigned long*)(&(m_pucCompressedListMem[ListNum][0])) = ListCnt;
		ChangeOrderLong(&(m_pucCompressedListMem[ListNum][0]));

		m_ulCompressedListSize[ListNum] = sizeof(ULONG);
/*
		for (i=0; i < ListCnt; i++) {
		
			TRACE("%s\n", &m_pucEntranceListMem[ListNum][sizeof(ULONG) + i * ucRecordSize]);
		}
*/
		// Liste mit -personalstamm- Daten komprimieren
		for (i=0; i < (ULONG)ListCnt; i++) {
			
			pPersonalStamm = (structPersonalStamm*)&m_pucCompressedListMem[ListNum][sizeof(ULONG) + i * sizeof(structPersonalStamm)];
			
			for (j=0; j < ucRecordSize; j++) {

				if (j < 13) {

					pPersonalStamm->pNr[j] = m_pucEntranceListMem[ListNum][sizeof(ULONG) + i * ucRecordSize + j];
				} 

				// Datensatz aufbereiten.
				if (m_pucEntranceListMem[ListNum][sizeof(ULONG) + i * ucRecordSize + j]) {
					// Abzug fuer ASCII - Werte
					pucRecord[j] = m_pucEntranceListMem[ListNum][sizeof(ULONG) + i * ucRecordSize + j] - 0x30;
				} else {
					pucRecord[j] = 0x00;
				}				
			}


			for (j=0; j < 13; j++) {
				
				if (!pPersonalStamm->pNr[j]) { // 0terminie suchen
				
					break;
				}
			}
			for (; j < 14; j++) { // rest mit 0 belegen
				
				pPersonalStamm->pNr[j] = 0x00;
			}

			pPersonalStamm->zeitmodell = pucRecord[14] * 10 + pucRecord[15];			
			pPersonalStamm->prio = pucRecord[17];

			// Strukturoffsets zu Listengroesse aufrechnen.
			m_ulCompressedListSize[ListNum] += sizeof(structPersonalStamm);
		}
	}
	
	// Liste Feiertage - Komprimieren
	if (ListNum == 3) {

		structFeiertag *pFeiertag;
		unsigned long i, j;
		unsigned char ucRecordSize = ENTRANCE_RECORD_SIZE_3;
		unsigned char pucRecord[ENTRANCE_RECORD_SIZE_3];

		// Anzahl Listenzeilen eintragen
		*(unsigned long*)(&(m_pucCompressedListMem[ListNum][0])) = ListCnt;
		ChangeOrderLong(&(m_pucCompressedListMem[ListNum][0]));

		m_ulCompressedListSize[ListNum] = sizeof(ULONG);

		// Liste mit -feiertage- Daten komprimieren
		for (i=0; i < (ULONG)ListCnt; i++) {
			
			for (j=0; j < ucRecordSize; j++) {

				// Datensatz aufbereiten.
				if (m_pucEntranceListMem[ListNum][sizeof(ULONG) + i * ucRecordSize + j]) {
					// Abzug fuer ASCII - Werte
					pucRecord[j] = m_pucEntranceListMem[ListNum][sizeof(ULONG) + i * ucRecordSize + j] - 0x30;
				} else {
					pucRecord[j] = 0x00;
				}
			}

			pFeiertag = (structFeiertag*)&m_pucCompressedListMem[ListNum][sizeof(ULONG) + i * sizeof(structFeiertag)];
			pFeiertag->zeitmodell = pucRecord[0] * 10 + pucRecord[1];
			// JJJJMMDD
			pFeiertag->datum = (ULONG)(((pucRecord[9] * 1000 + pucRecord[10] * 100 + pucRecord[11] * 10 + pucRecord[12]) << 16) | ((pucRecord[6] * 10 + pucRecord[7]) << 8) | (pucRecord[3] * 10 + pucRecord[4]));
			ChangeOrderLong((UCHAR*)&pFeiertag->datum);
			pFeiertag->prio = pucRecord[14];

			// Strukturoffsets zu Listengroesse aufrechnen.
			m_ulCompressedListSize[ListNum] += sizeof(structFeiertag);
		}
	}
	
	return TRUE;
}

void CAE3Buffer::QuicksortEntranceRecord(unsigned char ucNum, unsigned char ucOffs, long lLeftRecordNum, long lRightRecordNum)
{
	long lHelp1, lHelp2;

	if (lRightRecordNum > lLeftRecordNum) {

		//CString strTmp((const unsigned char*)&m_pucEntranceListMem[ucNum][lRightRecordNum * m_ucEntranceRecordSize[ucNum] + sizeof(ULONG) + ucOffs]);
		lHelp1 = lLeftRecordNum - 1;
		lHelp2 = lRightRecordNum;

		for (;;) {
		
			//while (CString((const unsigned char*)&m_pucEntranceListMem[ucNum][++lHelp1 * m_ucEntranceRecordSize[ucNum] + sizeof(ULONG) + ucOffs]) < strTmp);
			//while (CString((const unsigned char*)&m_pucEntranceListMem[ucNum][--lHelp2 * m_ucEntranceRecordSize[ucNum] + sizeof(ULONG) + ucOffs]) > strTmp);

			while (strcmp((const char*)&m_pucEntranceListMem[ucNum][++lHelp1 * m_ucEntranceRecordSize[ucNum] + sizeof(ULONG) + ucOffs], 
				          (const char*)&m_pucEntranceListMem[ucNum][lRightRecordNum * m_ucEntranceRecordSize[ucNum] + sizeof(ULONG) + ucOffs]) < 0);
			while (strcmp((const char*)&m_pucEntranceListMem[ucNum][--lHelp2 * m_ucEntranceRecordSize[ucNum] + sizeof(ULONG) + ucOffs], 
				          (const char*)&m_pucEntranceListMem[ucNum][lRightRecordNum * m_ucEntranceRecordSize[ucNum] + sizeof(ULONG) + ucOffs]) > 0);

			if (lHelp1 >= lHelp2) {
			
				break;
			}

			SwapEntranceRecord(ucNum, lHelp1, lHelp2);
		}

		SwapEntranceRecord(ucNum, lHelp1, lRightRecordNum);
		QuicksortEntranceRecord(ucNum, ucOffs, lLeftRecordNum, lHelp1 - 1);
		QuicksortEntranceRecord(ucNum, ucOffs, lHelp1 + 1, lRightRecordNum);
	}
}

void CAE3Buffer::SwapEntranceRecord(unsigned char ucNum, long lLeftRecordNum, long lRightRecordNum)
{
	int i;
	unsigned char ucTmp;
	for (i=0; i < m_ucEntranceRecordSize[ucNum]; i++) {
	
		ucTmp = m_pucEntranceListMem[ucNum][lLeftRecordNum * m_ucEntranceRecordSize[ucNum] + sizeof(ULONG) + i];
		m_pucEntranceListMem[ucNum][lLeftRecordNum * m_ucEntranceRecordSize[ucNum] + sizeof(ULONG) + i] = m_pucEntranceListMem[ucNum][lRightRecordNum * m_ucEntranceRecordSize[ucNum] + sizeof(ULONG) + i];
		m_pucEntranceListMem[ucNum][lRightRecordNum * m_ucEntranceRecordSize[ucNum] + sizeof(ULONG) + i] = ucTmp;
	}
}

void CAE3Buffer::ClearEntranceListBuffer(int iListNum)
{
	int i, j;
	if (iListNum == -1) { // Alle loeschen

		for (i=0; i < 4; i++) {
		
			m_ulEntranceListSize[i] = 0;
			m_ulCompressedListSize[i] = 0;
		}
	
		for (i=0; i < 4; i++) {

			for(j=0; j < FLASH_SIZE_LIST; j++) {

				m_pucEntranceListMem[i][j] = 0;
				m_pucCompressedListMem[i][j] = 0;
			}
		}

	} else {

		m_ulEntranceListSize[iListNum] = 0;
		m_ulCompressedListSize[iListNum] = 0;

		for (j=0; j < FLASH_SIZE_LIST; j++) {
		
			m_pucEntranceListMem[iListNum][j] = 0;
			m_pucCompressedListMem[iListNum][j] = 0;
		}
	}

}

void CAE3Buffer::CpyFromKonfigBuf(unsigned char *Des,unsigned char *Src,unsigned long Add,int Cnt)
{
	int i;

	ASSERT(Add >= FLASH_KONFIG_START);

	for (i=0; i < Cnt; i++) {
		
		Des[i] = Src[Add + i - FLASH_KONFIG_START];
	}
}

unsigned long CAE3Buffer::ChangeOrderULong(unsigned long val)
{
	unsigned char buf[4];

	*(unsigned long *)buf = val;
	ChangeOrderLong(buf);
	return *(unsigned long *)buf;
} 

int CAE3Buffer::GetTableCount(unsigned char ucList)
{
	if (ucList) {
		
		return MaxListB;
	} else {
		
		return MaxDatB;
	}
}

int CAE3Buffer::GetTableRecord(unsigned char ucNum,unsigned char *szName, 
		unsigned char *pucFieldCount, unsigned char *pucVariable, unsigned char ucList)
{
	int i;

	if (ucList) {

		// Auf Zutrtittskontrolllisten verzweigen
		if (ucNum >= 100 && ucNum < 104) {
			// Version 1
			*pucVariable = FALSE;
			return GetTableRecordEntrance(ucNum, szName, pucFieldCount);
		}
		
		if (ucNum >= MaxListB) {
			
			return FALSE;
		}
		
		for (i=0; i < (MAX_DS_NAME - 1) && 
			ListenBeschreibung[ucNum].Name[i] != 0; i++) {

			szName[i] = ListenBeschreibung[ucNum].Name[i];
		}
		szName[i] = 0;

		*pucFieldCount = ListenBeschreibung[ucNum].Felder;
		*pucVariable = FALSE;
	} else {

		if (ucNum >= MaxDatB) {
			
			return FALSE;
		}

		for (i=0; i < (MAX_DS_NAME - 1) && 
			DatensatzBeschreibung[ucNum].Name[i] != 0; i++) {

			szName[i] = DatensatzBeschreibung[ucNum].Name[i];
		}
		szName[i] = 0;

		*pucFieldCount = DatensatzBeschreibung[ucNum].Felder;
	}

	return 1;
}

int CAE3Buffer::GetTableField(unsigned char ucTableNum, unsigned char ucFieldNum, 
		unsigned char *szName, unsigned char *pucType, 
		unsigned char *pucLength, unsigned char ucList)
{
	int i;

	if (ucList) {

		// Auf Zutrittskontrolllisten verzweigen
		if (ucTableNum >= 100 && ucTableNum < 104) {
			// Version 1
			return GetTableFieldEntrance(ucTableNum, ucFieldNum, szName, pucType, pucLength);
		}
		
		if (ucTableNum >= MaxListB) {
			
			return FALSE;
		}

		if (ListenBeschreibung[ucTableNum].Felder <= ucFieldNum) {
			
			return FALSE;
		}

		for (i=0; i < (MAX_DS_NAME - 1) && 
			ListenFeld[ucTableNum][ucFieldNum].Name[i] != 0; i++) {

			szName[i] = ListenFeld[ucTableNum][ucFieldNum].Name[i];
		}
		szName[i] = 0;

		*pucType   = ListenFeld[ucTableNum][ucFieldNum].Typ;
		*pucLength = ListenFeld[ucTableNum][ucFieldNum].Cnt;
	} else {

		if (ucTableNum >= MaxDatB) {
			
			return FALSE;
		}

		if (DatensatzBeschreibung[ucTableNum].Felder <= ucFieldNum) {
			
			return FALSE;
		}

		for (i=0; i < (MAX_DS_NAME - 1) && 
			DatensatzFeld[ucTableNum][ucFieldNum].Name[i] != 0; i++) {

			szName[i] = DatensatzFeld[ucTableNum][ucFieldNum].Name[i];
		}
		szName[i] = 0;

		*pucType   = DatensatzFeld[ucTableNum][ucFieldNum].Typ;
		*pucLength = DatensatzFeld[ucTableNum][ucFieldNum].Cnt;
	}

	return 1;
}

int CAE3Buffer::GetTableRecordEntrance(unsigned char ucNum,unsigned char *szName, 
		unsigned char *pucFieldCount)
{
	switch(ucNum - 100) {
	case 0:
		strcpy((char*)szName, "Zeitzonen");
		*pucFieldCount = 6;
		break;
	case 1:
		strcpy((char*)szName, "Zeitmodelle");
		*pucFieldCount = 16;
		break;
	case 2:
		strcpy((char*)szName, "Personalstamm");
		*pucFieldCount = 3;
		break;
	case 3:
		strcpy((char*)szName, "Feiertage");
		*pucFieldCount = 5;
		break;
	default:
		return FALSE;
	}

	return TRUE;
}

int CAE3Buffer::GetTableFieldEntrance(unsigned char ucTableNum, unsigned char ucFieldNum, 
		unsigned char *szName, unsigned char *pucType, 
		unsigned char *pucLength)
{
	switch(ucTableNum - 100) {
	case 0: // Zeitzonen
		switch(ucFieldNum) {
		case 0:
			strcpy((char*)szName, "WTAG");
			*pucType   = 3;
			*pucLength = 8;
			break;
		case 1:
			strcpy((char*)szName, "Start (h)");
			*pucType   = 3;
			*pucLength = 3;
			break;
		case 2:
			strcpy((char*)szName, "Start (min)");
			*pucType   = 3;
			*pucLength = 3;
			break;
		case 3:
			strcpy((char*)szName, "Ende (h)");
			*pucType   = 3;
			*pucLength = 3;
			break;
		case 4:
			strcpy((char*)szName, "Ende (min)");
			*pucType   = 3;
			*pucLength = 3;
			break;
		case 5:
			strcpy((char*)szName, "Aktiv");
			*pucType   = 3;
			*pucLength = 2;
			break;
		}
		break;
	case 1: // Zeitmodelle
		switch(ucFieldNum) {
		case 0:
			strcpy((char*)szName, "BGR");
			*pucType   = 3;
			*pucLength = 3;
			break;
		case 1:	case 2:	case 3:	case 4:	case 5:	case 6:	case 7:	case 8:	
		case 9:	case 10: case 11: case 12: case 13: case 14: case 15: case 16:
			sprintf((char*)szName, "ZZ%i", ucFieldNum);
			*pucType   = 3;
			*pucLength = 3;
			break;
		default:
			return FALSE;
		}
		break;
	case 2: // Personalstamm
		switch(ucFieldNum) {
		case 0:
			strcpy((char*)szName, "Ausweis_Nr");
			*pucType   = 4;
			*pucLength = 14;
			break;
		case 1:
			strcpy((char*)szName, "BGR");
			*pucType   = 3;
			*pucLength = 3;
			break;
		case 2:
			strcpy((char*)szName, "Prio");
			*pucType   = 3;
			*pucLength = 2;
			break;
		default:
			return FALSE;
		}
		break;
	case 3: // Feiertage
		switch(ucFieldNum) {
		case 0:
			strcpy((char*)szName, "BGR");
			*pucType   = 3;
			*pucLength = 3;
			break;
		case 1:
			strcpy((char*)szName, "Tag");
			*pucType   = 3;
			*pucLength = 3;
			break;
		case 2:
			strcpy((char*)szName, "Monat");
			*pucType   = 3;
			*pucLength = 3;
			break;
		case 3:
			strcpy((char*)szName, "Jahr");
			*pucType   = 3;
			*pucLength = 5;
			break;
		case 4:
			strcpy((char*)szName, "Prio");
			*pucType   = 3;
			*pucLength = 2;
			break;
		default:
			return FALSE;
		}
		break;
	default:
		return FALSE;
	}

	return TRUE;
}

// ########################################################
// Alte Datensatzfunktionen
#ifdef _WIN32
// Zerlegt den Buffer zu einzelnen Datensaetzen und
// fuegt diese dem DatenArray hinzu
// Buffer - Zeiger auf das erste Nutzdatenbyte
// length - Laenge der Nutzdaten
int CAE3Buffer::AppendData(unsigned char *pucBuffer, int iLength)
{
	int DatasetLength, index = 0;

	while(index < iLength) {

		DatasetLength = pucBuffer[index] - 2;
		ASSERT(DatasetLength >= 10);

		m_DataList.AddData(&(pucBuffer[index + 9]), DatasetLength - 9);
		index += DatasetLength;
	}

	//CFile file;  // nur fuer DEBUG Zwecke
	//file.Open("Temp",CFile::modeCreate|CFile::modeWrite);
	//file.Write(Buffer,length);
	
	return FALSE;
}
#endif

void CAE3Buffer::DefaultIOModule(TAE3IOModule *pIOModule)
{
	int i;
	pIOModule->ucActiveU = 0x00;
	pIOModule->ucOnScreenU = 0x00;
	pIOModule->ucActiveI = 0x00;
	pIOModule->ucOnScreenI = 0x00;

	// Setzten der default Werte.
	for (i=0; i < 8; i++) {
		// Spannungskanaele
		sprintf((char*)pIOModule->szNameU[i], "U%i", i + 1);
		sprintf((char*)pIOModule->szUnitU[i], "V");
		pIOModule->fMinU[i] = 0;
		pIOModule->fMaxU[i] = 10;

		// Stomkanaele
		sprintf((char*)pIOModule->szNameI[i], "I%i", i + 1);
		sprintf((char*)pIOModule->szUnitI[i], "mA");
		pIOModule->fMinI[i] = 0;
		pIOModule->fMaxI[i] = 20;
	}
}

// Groesse der Setupdaten im Flash ermitteln
ULONG CAE3Buffer::GetFlashSetupSize()
{
	// ##############################################################
	// Pointer sind direkte Adressen die im AEIII+:
	//   1. Im Motorolla - Format vorliegen.
	//   2. Mit einem Basisoffset -FLASH_KONFIG_START- berechnet sind.

	// Pointer auf ListenBeschreibungen ermitteln
	ULONG pLB = *(ULONG*)(&m_pucEepMem[P_LIST_STRUCT]);

	//if (App->ByteOrder == BO_MOT) {
		
		ChangeOrderLong((UCHAR*)&pLB);
	//}
	pLB -= FLASH_KONFIG_START;

	// Pointerarray der Listenbeschreibungen durchschreiten
	int i;
	for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {
	
		// Nullpointer suchen fuer Listenanzahl
		if (*(ULONG*)&m_pucFlashMem[pLB + i * sizeof(ULONG)] == 0ul) {
		
			break;
		}
	}

	// Liste gefunden?
	if (i > 0) {
	
		// Pointer auf letzte definierte Liste
		pLB = *(ULONG*)(&m_pucFlashMem[pLB + (i - 1) * sizeof(ULONG)]);

		//if (App->ByteOrder == BO_MOT) {
			
			ChangeOrderLong((UCHAR*)&pLB);
		//}
		pLB -= FLASH_KONFIG_START;

		TDatensatzBeschreibung *pLBStruct = (TDatensatzBeschreibung*)(&m_pucFlashMem[pLB]);

		// Pointer auf Feldarray der Liste
		ULONG pFeldArray = pLBStruct->P;
		//if (App->ByteOrder == BO_MOT) {
			
			ChangeOrderLong((UCHAR*)&pFeldArray);
		//}
		pFeldArray -= FLASH_KONFIG_START;

		// Felder vorhanden?
		if (pLBStruct->Felder) {
		
			// Pointer auf letztes Feld der Listenbeschreibung ermitteln.
			ULONG pFeld = *(ULONG*)&m_pucFlashMem[pFeldArray + (pLBStruct->Felder - 1) * sizeof(ULONG)];
			//if (App->ByteOrder == BO_MOT) {
				
				ChangeOrderLong((UCHAR*)&pFeld);
			//}
			pFeld -= FLASH_KONFIG_START;
			
			// Feldadresse + eigene Groesse bildet FlashSetupSize
			return pFeld + sizeof(TDatensatzFeld);
		}


		// Keine Felder definiert!
		return pFeldArray;
	}
	
	// Keine Listenbeschreibung definiert!
	// Die Anzahl fuer Listenbeschreibungspointer muss mitberechnet werden,
	// um bei einer Uebertragung diese zu loeschen.
	return pLB + MAX_DATENSATZ_BESCHREIBUNG * sizeof(ULONG);
}

ULONG CAE3Buffer::GetMDEEKFlashSize_0400()
{
	// Erweiterten Struktur lesen
	int i;
	TAE3Extended extBegin;
	for (i=0; i < sizeof(TAE3Extended); i++) {
	
		((UCHAR*)&extBegin)[i] = m_pucExtendedMem[i];
	}
	
	// Erweiterte Struktur vorhanden?
	if (extBegin.ulCheck != EXTENDED_CHECK) {
	
		return 0;
	}

	// Strukturen und Eingabeketten fuer MDE lesen.
	if (extBegin.pulFreeScope == 1) {
	
		// Kein Eintrag bei neuem Dokument. Erst nach MkFlashImage
		// 1Byte Anzahl Eingabeketten fuer MDE
		return 1;
	
	} else if (extBegin.pulFreeScope != 0) {

		// Nicht gerade mit DLL ausgelesen.
		// Groesse Eingabeketten fuer MDE
		return extBegin.pulFreeScope - FLASH_KONFIG_START;
	}

	int Unterpunkte;
	ULONG pEK;

	Unterpunkte = m_pucExtendedMem[extBegin.pulEKArray];
	if (Unterpunkte == 0) {
	
		// 1Byte Anzahl Eingabeketten fuer MDE
		return 1;
	}
	
	// Pointer der letzten Eingabekette ermitteln
	pEK = extBegin.pulEKArray + sizeof(UCHAR) + (Unterpunkte - 1) * sizeof(ULONG);	
	pEK = *(ULONG*)(&m_pucExtendedMem[pEK]);
	//if (App->ByteOrder == BO_MOT) {
		
		ChangeOrderLong((unsigned char*)&pEK);
	//}
	// 16kByte fuer FlashOffset + Basisadresse fuer Konfigdaten subtrahieren
	pEK -= (FLASH_SIZE_SU + FLASH_KONFIG_START);

	// vor jeder Eingabekettenstruktur liegt eine Bedien - Struktur
	pEK += sizeof(TBedien);
	TBDatensatzEinlesen *pDE = (TBDatensatzEinlesen*)&m_pucExtendedMem[extBegin.pulEKArray + pEK];

	// Pointer auf Feldarray der Eingabekette
	ULONG pFeldArray = pDE->P;
	//if (App->ByteOrder == BO_MOT) {
		
		ChangeOrderLong((UCHAR*)&pFeldArray);
	//}
	pFeldArray -= (FLASH_SIZE_SU + FLASH_KONFIG_START);

	// Felder vorhanden?
	if (pDE->Felder > 0) {

	
		// Pointer auf letztes Feld der Eingabekette ermitteln.
		ULONG pFeld = *(ULONG*)&m_pucExtendedMem[extBegin.pulEKArray + pFeldArray + (pDE->Felder - 1) * sizeof(ULONG)];
		//if (App->ByteOrder == BO_MOT) {
			
			ChangeOrderLong((UCHAR*)&pFeld);
		//}
		pFeld -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
		
		//TBFeldEinlesen *pDF = (TBFeldEinlesen*)&m_pucExtendedMem[extBegin.pulEKArray + pFeld];
		
		// Feldadresse + eigene Groesse bildet FlashSetupSize
		return pFeld + sizeof(TBFeldEinlesen);
	}

	// Keine Eingabekettenfelder definiert!
	return pFeldArray;
}

ULONG CAE3Buffer::GetMDEEKFlashSize()
{
	// Erweiterten Struktur lesen
	int i;
	TAE3Extended extBegin;
	for (i=0; i < sizeof(TAE3Extended); i++) {
	
		((UCHAR*)&extBegin)[i] = m_pucExtendedMem[i];
	}
	
	// Erweiterte Struktur vorhanden?
	if (extBegin.ulCheck != EXTENDED_CHECK) {
	
		return 0;
	}

	// Strukturen und Eingabeketten fuer MDE lesen.
	if (extBegin.pulFreeScope == 1) {
	
		// Kein Eintrag bei neuem Dokument. Erst nach MkFlashImage
		// 4Byte Anzahl Eingabeketten fuer MDE
		return 4;
	
	} else if (extBegin.pulFreeScope != 0) {

		// Nicht gerade mit DLL ausgelesen.
		// Groesse Eingabeketten fuer MDE
		return extBegin.pulEntranceTables - extBegin.pulEKArray;
	}

	int Unterpunkte;
	ULONG pEK;

	Unterpunkte = m_pucExtendedMem[extBegin.pulEKArray];
	if (Unterpunkte == 0) {
	
		// 4Byte Anzahl Eingabeketten fuer MDE
		return 4;
	}
	
	// Pointer der letzten Eingabekette ermitteln
	pEK = extBegin.pulEKArray + sizeof(ULONG) + (Unterpunkte - 1) * sizeof(ULONG);	
	pEK = *(ULONG*)(&m_pucExtendedMem[pEK]);
	//if (App->ByteOrder == BO_MOT) {
		
		ChangeOrderLong((unsigned char*)&pEK);
	//}
	// 16kByte fuer FlashOffset + Basisadresse fuer Konfigdaten subtrahieren
	pEK -= (FLASH_SIZE_SU + FLASH_KONFIG_START);

	// vor jeder Eingabekettenstruktur liegt eine Bedien - Struktur
	pEK += sizeof(TBedien);
	TBDatensatzEinlesen *pDE = (TBDatensatzEinlesen*)&m_pucExtendedMem[extBegin.pulEKArray + pEK];

	// Pointer auf Feldarray der Eingabekette
	ULONG pFeldArray = pDE->P;
	//if (App->ByteOrder == BO_MOT) {
		
		ChangeOrderLong((UCHAR*)&pFeldArray);
	//}
	pFeldArray -= (FLASH_SIZE_SU + FLASH_KONFIG_START);

	// Felder vorhanden?
	if (pDE->Felder > 0) {

	
		// Pointer auf letztes Feld der Eingabekette ermitteln.
		ULONG pFeld = *(ULONG*)&m_pucExtendedMem[extBegin.pulEKArray + pFeldArray + (pDE->Felder - 1) * sizeof(ULONG)];
		//if (App->ByteOrder == BO_MOT) {
			
			ChangeOrderLong((UCHAR*)&pFeld);
		//}
		pFeld -= (FLASH_SIZE_SU + FLASH_KONFIG_START);
		
		//TBFeldEinlesen *pDF = (TBFeldEinlesen*)&m_pucExtendedMem[extBegin.pulEKArray + pFeld];
		
		// Feldadresse + eigene Groesse bildet FlashSetupSize
		return pFeld + sizeof(TBFeldEinlesen);
	}

	// Keine Eingabekettenfelder definiert!
	return pFeldArray;
}

ULONG CAE3Buffer::GetEntranceEKFlashSize()
{
	// Erweiterten Struktur lesen
	int i;
	TAE3Extended extBegin;
	for (i=0; i < sizeof(TAE3Extended); i++) {
	
		((UCHAR*)&extBegin)[i] = m_pucExtendedMem[i];
	}
	
	// Erweiterte Struktur vorhanden?
	if (extBegin.ulCheck != EXTENDED_CHECK) {
	
		return 0;
	}

	// Strukturen und Eingabeketten fuer Zutritt lesen.
	if (extBegin.pulFreeScope == 1) {
	
		// Kein Eintrag bei neuem Dokument. Erst nach MkFlashImage
		// 4Byte Anzahl Eingabeketten fuer Zutritt
		return 4;
	
	} else if (extBegin.pulFreeScope != 0) {

		// Nicht gerade mit DLL ausgelesen.
		// Groesse Eingabeketten fuer Zutritt
		return extBegin.pulFreeScope - extBegin.pulEKArrayEntrance;
	}

	int Unterpunkte;
	ULONG pEK;

	Unterpunkte = m_pucExtendedMem[extBegin.pulEKArrayEntrance];
	if (Unterpunkte == 0) {
	
		// 4Byte Anzahl Eingabeketten fuer Zutritt
		return 4;
	}
	
	// Pointer der letzten Eingabekette ermitteln
	pEK = extBegin.pulEKArrayEntrance + sizeof(ULONG) + (Unterpunkte - 1) * sizeof(ULONG);	
	pEK = *(ULONG*)(&m_pucExtendedMem[pEK]);
	//if (App->ByteOrder == BO_MOT) {
		
		ChangeOrderLong((unsigned char*)&pEK);
	//}
	// 32kByte fuer FlashOffset + Basisadresse fuer Konfigdaten subtrahieren
	pEK -= (FLASH_SIZE_SU + EXTENDED_SIZE_SU + FLASH_KONFIG_START);

	// vor jeder Eingabekettenstruktur liegt eine Bedien - Struktur
	pEK += sizeof(TBedien);
	TBDatensatzEinlesen *pDE = (TBDatensatzEinlesen*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance + pEK];

	// Pointer auf Feldarray der Eingabekette
	ULONG pFeldArray = pDE->P;
	//if (App->ByteOrder == BO_MOT) {
		
		ChangeOrderLong((UCHAR*)&pFeldArray);
	//}
	pFeldArray -= (FLASH_SIZE_SU + EXTENDED_SIZE_SU + FLASH_KONFIG_START);

	// Felder vorhanden?
	if (pDE->Felder > 0) {

	
		// Pointer auf letztes Feld der Eingabekette ermitteln.
		ULONG pFeld = *(ULONG*)&m_pucExtendedMem[extBegin.pulEKArrayEntrance + pFeldArray + (pDE->Felder - 1) * sizeof(ULONG)];
		//if (App->ByteOrder == BO_MOT) {
			
			ChangeOrderLong((UCHAR*)&pFeld);
		//}
		pFeld -= (FLASH_SIZE_SU + EXTENDED_SIZE_SU + FLASH_KONFIG_START);
		
		//TBFeldEinlesen *pDF = (TBFeldEinlesen*)&m_pucExtendedMem[extBegin.pulEKArray + pFeld];
		
		// Feldadresse + eigene Groesse bildet FlashSetupSize
		return pFeld + sizeof(TBFeldEinlesen);
	}

	// Keine Eingabekettenfelder definiert!
	return pFeldArray;
}

