/**
 * @file  Channel.cpp
 * @brief Implementierung der Klasse CChannel.
 */
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Channel.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

// ##################################################################
/**
 * @brief Initialisieren der Membervariablen.
 *
 * @param ucNum Nummer der Kanalklasse fuer z.B. Log.
 */
CChannel::CChannel(unsigned char ucNum)
{
	// Kanalnummer zuweisen
	m_ucNum = ucNum;

	// Membervars initialisieren
	m_ChannelType    = Undef;
	m_hCom           = INVALID_HANDLE_VALUE;
	m_hSocket        = INVALID_SOCKET;
	m_bLogIsOn       = false;
	sprintf(m_szLogFileName, COM_LOG_FILENAME, ucNum);
	m_Com.SetLogFileName((const char*)m_szLogFileName);
	
	// Eigener Handle erzeugt?
	m_bOwnHandle     = false;
}

// ##################################################################
/**
 * @brief Destruktor schliesst die eventuell noch geoeffnete Schnittstelle. 
 *
 */
CChannel::~CChannel() 
{
	// Schliessen der Schnittstelle, wenn nicht bereits.
	Close();
}

//////////////////////////////////////////////////////////////////////
// Members
//////////////////////////////////////////////////////////////////////

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
UCHAR CChannel::GetChannelNum()
{
	return m_ucNum;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel::ClearCom()
{
	unsigned char pucBuf[0x100];
	unsigned long ulReadCount = 0xFF;

	if (m_ChannelType == Class) {
		
		m_Com.ClearCom();
		return;
	}

	// eventuell anstehende Datenpakete verwerfen
	for (int i=0; (i < 3) && ulReadCount; i++) {

		if (!Read(pucBuf, 0xFF, &ulReadCount)) {
			
			break;
		}
	}
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel::SetLogFileName(const char *szLogFileName)
{
	strcpy(m_szLogFileName, (const char*)szLogFileName);
	m_Com.SetLogFileName(szLogFileName);
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
#ifndef TRACE_DEBUG
void CChannel::ComLog(const char *Format, ...)
{
	va_list ap;
	FILE *Logfp;

	if (m_bLogIsOn) {
		va_start(ap, Format);

		if ((Logfp = fopen(m_szLogFileName, "a")) == (FILE *)0) {

			return;
		}
		vfprintf(Logfp, Format, ap);

		va_end(ap);
		fclose(Logfp);
	}
}
#endif

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
int CChannel::ComUseHandle(HANDLE hnd) 
{
	if (m_ChannelType != Undef) {

		return FALSE;
	}

	m_hCom        = hnd;
	m_ChannelType = Serial;

	return TRUE;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
int CChannel::ComReleaseHandle()
{
	if (m_ChannelType != Serial) {

		return FALSE;
	}

	m_hCom        = INVALID_HANDLE_VALUE;
	m_ChannelType = Undef;

	return TRUE;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
HANDLE CChannel::GetComPort()
{
	HANDLE hCom = INVALID_HANDLE_VALUE;

	if (m_ChannelType == Serial && m_hCom != INVALID_HANDLE_VALUE) {

		hCom = m_hCom;
	} else if (m_ChannelType == Socket && m_hSocket != INVALID_SOCKET) {

		hCom = (HANDLE)m_hSocket;
	} else if (m_ChannelType == Class && m_Com.hcom != INVALID_HANDLE_VALUE) {
	
		hCom = m_Com.hcom;
	} else if (m_ChannelType == Class && m_Com.Socket != INVALID_SOCKET) {
	
		hCom = (HANDLE)m_Com.Socket;
	}

	return hCom;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
int CChannel::SetComPort(HANDLE hCom, int iType, int iComValue, int iTimeOut)
{
	switch (iType) {
	case 0:
		// Muss undefiniert sein oder mit gewuenschtem uebereinstimmen
		if ((m_ChannelType == Undef) || 
			((m_ChannelType == Serial) && (m_hCom == hCom)) ) {

			m_iComPortTimeOut = iTimeOut;

			if (!SetSerialComState(hCom, iComValue, iTimeOut)) {

				return FALSE;
			}

			m_hCom        = hCom;
			m_ChannelType = Serial;
		} else {
		
			return FALSE;
		}
		break;

	case 1:
		// Muss undefiniert sein oder mit gewuenschtem uebereinstimmen
		if ((m_ChannelType == Undef) || 
			((m_ChannelType == Socket) && (m_hSocket == (SOCKET)hCom)) ) {

			m_iComPortTimeOut = iTimeOut;
			m_hSocket         = (SOCKET)hCom;
			m_ChannelType     = Socket;
		} else {
		
			return FALSE;
		}
		break;

	case 2:
		// Muss undefiniert sein oder mit gewuenschtem uebereinstimmen
		if ((m_ChannelType == Undef) ||
			((m_ChannelType == Class) && (m_Com.hcom == hCom)) ) {

			m_iComPortTimeOut = iTimeOut;

			if (!m_Com.SetSerialComState(hCom, iComValue, iTimeOut)) {

				return FALSE;
			}

			strcpy((char*)m_Com.ComPort, "");
			m_Com.hcom    = hCom;
			m_Com.Baud    = iComValue;

			strcpy(m_Com.RemoteIP, "");
			m_Com.Socket  = INVALID_SOCKET;
			m_Com.PortNo  = 0;

			m_Com.AdAbsender = 0;
			m_Com.Typ        = T_S2_RS232;
			m_Com.ComTimeOut = iTimeOut;

			m_ChannelType = Class;

		} else {
		
			return FALSE;
		}
		break;
	case 3:
		// Muss undefiniert sein oder mit gewuenschtem uebereinstimmen
		if ((m_ChannelType == Undef) ||
			((m_ChannelType == Class) && (m_Com.Socket == (SOCKET)hCom)) ) {

			m_iComPortTimeOut = iTimeOut;

			strcpy((char*)m_Com.ComPort, "");
			m_Com.hcom   = INVALID_HANDLE_VALUE;
			m_Com.Baud   = 0;

			strcpy(m_Com.RemoteIP, "");
			m_Com.Socket = (SOCKET)hCom;
			m_Com.PortNo = iComValue;

			m_Com.AdAbsender = 0;
			m_Com.Typ        = T_S2_TCP_IP;
			m_Com.ComTimeOut = iTimeOut;

			m_ChannelType = Class;

		} else {
		
			return FALSE;
		}
		break;
	}

	return TRUE;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
int CChannel::SetSerialComState(HANDLE hComPort, int iBaudRate, int iTimeOut)
{
#ifdef _WIN32
	DCB          dcb;
	COMMTIMEOUTS ctout;
	DWORD        cerr = 0;

	memset(&dcb, 0, sizeof(dcb));
	dcb.DCBlength = sizeof(dcb);
	dcb.fBinary   = 1;

	dcb.BaudRate  = iBaudRate;
	dcb.ByteSize  = 8;
	dcb.Parity    = NOPARITY;
	dcb.StopBits  = ONESTOPBIT;


	if (!SetCommState(hComPort, &dcb)) {

		CloseHandle(hComPort);
		return FALSE;
	}

	ctout.ReadIntervalTimeout         = 0;
	ctout.ReadTotalTimeoutMultiplier  = 0;
	ctout.ReadTotalTimeoutConstant    = (DWORD) iTimeOut;
	ctout.WriteTotalTimeoutConstant   = 0;
	ctout.WriteTotalTimeoutMultiplier = 0;

	if (!SetCommTimeouts(hComPort, &ctout)) {

		CloseHandle(hComPort);
		return FALSE;
	}

	// unsolved: fuer modems + RS485 notwendig !?!
	if (!EscapeCommFunction(hComPort, SETRTS)) {

		CloseHandle(hComPort);
		return FALSE;
	}
	if (!EscapeCommFunction(hComPort, SETDTR)) {

		CloseHandle(hComPort);
		return FALSE;
	}

	if (!ClearCommError(hComPort, &cerr, NULL)) {

		CloseHandle(hComPort);
		return FALSE;
	}

#else
	int rc;
	int baudmode, charmode, paricheck, stopmode;
	int termio_cm;

	switch(iBaudRate) {
	case 9600:
		baudmode = B9600;
		break;
	case 19200:
		baudmode = B19200;
	case 38400:
		baudmode = B38400;
		break;
	default:
		return FALSE;
	}

	charmode = CS8;
	paricheck = 0000000;
	stopmode = 0000000;

	termio_s.c_iflag = 0;
	termio_s.c_oflag = 0;
	termio_s.c_cflag = 0;
	termio_s.c_lflag = 0;
	termio_s.c_line  = 0;
	(void) memset((void *)termio_s.c_cc, 0, NCC);
   
	termio_s.c_cflag |= HUPCL;        // hangup line when close/exit(DTR of)
	termio_s.c_cflag |= CLOCAL;       // local/direct connection
   
	termio_s.c_cflag |= baudmode;     // set baudrate
	termio_s.c_cflag |= charmode;     // set databits
	termio_s.c_cflag |= paricheck;    // set parity
	termio_s.c_cflag |= stopmode;     // set stopbits

	termio_s.c_cflag |= CREAD;        // enable input queue

	termio_s.c_cc[VMIN]  = 1;         // read waiting for 1 char
   
	if ((rc = ioctl(hComPort, TCSETA, &termio_s)) < 0) {
		ComLog("SetSerialComState.ioctl: failed (rc=%d) (errno=%d)\n", rc, errno);
		return FALSE;
	}

	if ((rc = ioctl(hComPort, TCFLSH, TCIOFLUSH)) < 0) {
		ComLog("SetSerialComState.ioctl: failed (rc=%d) (errno=%d)\n", rc, errno);
		return FALSE;
	}

	if ((rc = ioctl(hComPort, TIOCMGET, &termio_cm)) < 0) {
		ComLog("SetSerialComState.ioctl: failed (rc=%d) (errno=%d)\n", rc, errno);
		return FALSE;
	}

	if (! (termio_cm & TIOCM_RTS)) termio_cm |= TIOCM_RTS;
	if (! (termio_cm & TIOCM_DTR)) termio_cm |= TIOCM_DTR;

	if ((rc = ioctl(hComPort, TIOCMSET, &termio_cm)) < 0) {
		ComLog("SetSerialComState.ioctl: failed (rc=%d) (errno=%d)\n", rc, errno);
		return FALSE;
	}
#endif

	return TRUE;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
int CChannel::SerialOpen(char *szComPort, int iBaudRate, int iTimeOut)
{
	int rc;

	ComLog("SerialOpen.Begin\n");
	
	m_iComPortTimeOut = iTimeOut;

#ifdef _WIN32

	if ((m_hCom = CreateFile(
				szComPort,						// pointer to name of the file 
				(GENERIC_READ | GENERIC_WRITE),	// access (read-write) mode 
				0,								// share mode 
				NULL,							// pointer to security descriptor 
				OPEN_EXISTING,					// how to create 
				0,								// file attributes 
				0 								// handle to file with attributes to copy  
				)) == INVALID_HANDLE_VALUE) { 

		return FALSE;
	}
#else
	errno = 0;

	if ((rc = open(szComPort, O_RDWR|O_NDELAY)) < 0) {
		ComLog("SerialOpen.open: failed (rc=%d) (errno=%d)\n", rc, errno);
		return FALSE;
	}

	m_hCom = rc;

	if ((rc = fcntl(m_hCom, F_SETFL, O_RDWR)) < 0) {
		ComLog("SerialOpen.fcntl: failed (rc=%d) (errno=%d)\n", rc, errno);
		return FALSE;
	}

	flock_s.l_type    = F_WRLCK;  // lock stream for read and write
	flock_s.l_whence  = 0;        // lock whole file -> 0,0,0
	flock_s.l_start   = 0;
	flock_s.l_len     = 0;

	if ((rc = fcntl(m_hCom, F_SETLK, &flock_s)) < 0) {
		ComLog("SerialOpen.fcntl: failed (rc=%d) (errno=%d)\n", rc, errno);
		return FALSE;
	}
#endif

	rc = SetSerialComState(m_hCom, iBaudRate, iTimeOut);

	ComLog("SerialOpen.End\n");
	
	return rc;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel::SerialClose(void)
{
	ComLog("SerialClose.Begin\n");

#ifdef _WIN32
	EscapeCommFunction(m_hCom,CLRRTS);
	EscapeCommFunction(m_hCom,CLRDTR);
	CloseHandle(m_hCom);
#else
	int rc;
	int termio_cm;

	if ((rc = ioctl(m_hCom, TIOCMGET, &termio_cm)) < 0) {
		ComLog("SerialClose.ioctl: failed (rc=%d) (errno=%d)\n", rc, errno);
	}

	if ((termio_cm & TIOCM_RTS)) termio_cm &= ~TIOCM_RTS;
	if ((termio_cm & TIOCM_DTR)) termio_cm &= ~TIOCM_DTR;

	if ((rc = ioctl(m_hCom, TIOCMSET, &termio_cm)) < 0) {
		ComLog("SerialClose.ioctl: failed (rc=%d) (errno=%d)\n", rc, errno);
	}

	close(m_hCom);
#endif

	m_hCom = INVALID_HANDLE_VALUE;

	ComLog("SerialClose.End\n");
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
int CChannel::SerialRead(char *pcInputBuf, int iInputLen, unsigned long *pdwReadLen)
{
	*pdwReadLen = 0;

	ComLog("SerialRead.Begin: (%d):(%d)\n", iInputLen, m_iComPortTimeOut);

#ifndef _WIN32
	int rc;
	struct timeval seltime;
	fd_set readfds;

	FD_ZERO((fd_set FAR *) &readfds);
	FD_SET(m_hCom, (fd_set FAR *) &readfds);
	seltime.tv_sec  = m_iComPortTimeOut / 1000;
	seltime.tv_usec = (m_iComPortTimeOut % 1000) * 1000;

	ComLog("SerialRead.select_0: (%d) (%d:%d)\n", rc,
		seltime.tv_sec, seltime.tv_usec);

	if ((rc = select(FD_SETSIZE,
		(fd_set FAR *)&readfds, (fd_set FAR *) 0, (fd_set FAR *) 0,
		(struct timeval *)&seltime)) < 0) {

		ComLog("SerialRead.select: failed(%d)\n", WSAGetLastError());
		return FALSE;
	}

	ComLog("SerialRead.select: (%d) (%d:%d)\n", rc,
		seltime.tv_sec, seltime.tv_usec);

	if (rc == 0) { // No Inputdata nach ComPortTimeOut millis

		return FALSE;
	}
#endif

#ifdef _WIN32
	if (!ReadFile(m_hCom, pcInputBuf, iInputLen, pdwReadLen, NULL)) {
		ComLog("SerialRead.ReadFile: failed\n");
		return FALSE;
	}
#else
	errno = 0;

	if ((rc = read(m_hCom, pcInputBuf, iInputLen)) < 0) {
		ComLog("SerialRead.read: failed (rc=%d) (errno=%d)\n", rc, errno);
		return FALSE;
	}
	*pdwReadLen = rc;
#endif

	ComLog("SerialRead.read: (%d)\n", *pdwReadLen);
	ComLog("SerialRead.End\n");
	
	return TRUE;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
int CChannel::SerialWrite(const char *pcOutputBuf, int iOutputLen, unsigned long *pdwWriteLen)
{
	ComLog("\nSerialWrite.Begin: (%d)\n", iOutputLen);
	
#ifdef _WIN32
	if (!WriteFile(m_hCom, pcOutputBuf, iOutputLen, pdwWriteLen, NULL)) {
		ComLog("SerialWrite.WriteFile: failed\n");
		return FALSE;
	}
#else
	int rc;
	errno = 0;

	if ((rc = write(m_hCom, pcOutputBuf, iOutputLen)) < 0) {
		ComLog("SerialWrite.write: failed (rc=%d) (errno=%d)\n", rc, errno);
		return FALSE;
	}
	*pdwWriteLen = rc;
#endif

	ComLog("SerialWrite.write: (%d)\n", *pdwWriteLen);
	ComLog("SerialWrite.End\n");
	
	return TRUE;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
int CChannel::SocketOpen(char *szRemoteIP, int iPortNum, int iTimeOut)
{
	SOCKADDR_IN sin;
	memset(&sin, 0, sizeof(sin));
   	sin.sin_family = AF_INET;
	sin.sin_port = htons(iPortNum);
	sin.sin_addr.s_addr = inet_addr(szRemoteIP);

	m_iComPortTimeOut = iTimeOut;

	ComLog("SocketOpen.Begin: (%s:%d)\n", szRemoteIP, iPortNum);

	if (m_hSocket != INVALID_SOCKET) {

		ComLog("SocketOpen: Socket schon in Verwendung.\n");
		return FALSE;
	}

	if (sin.sin_addr.s_addr == INADDR_NONE)	{ // Adresse ueber DNS aufloesen

		HOSTENT *he;
		he = gethostbyname(szRemoteIP);
		if (! he) {

			ComLog("SocketOpen.gethostbyname: failed(%d)\n", WSAGetLastError());
			return FALSE;
		}

		sin.sin_addr.s_addr = *((unsigned long*)he->h_addr);
	}

	if ((m_hSocket = socket(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET) {

		ComLog("SocketOpen.socket: failed(%d)\n", WSAGetLastError());
		return FALSE;
	}
      
	if (connect(m_hSocket,(SOCKADDR*)&sin,sizeof(sin)) == SOCKET_ERROR) {

		SocketClose();
		ComLog("SocketOpen.connect: failed(%d)\n", WSAGetLastError());
		return FALSE;
	}

	ComLog("SocketOpen.End\n");

	return TRUE;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel::SocketClose(void)
{
	ComLog("SocketClose.Begin\n");

#ifdef _WIN32
	closesocket(m_hSocket);
#else
	shutdown(m_hSocket, 2);
	close(m_hSocket);
#endif

	m_hSocket = INVALID_SOCKET;

	ComLog("SocketClose.End\n");
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
int CChannel::SocketRead(char *pcInputBuf, int iInputLen, unsigned long *pdwReadLen)
{
	int rc;

	struct timeval seltime;
	fd_set readfds;

	*pdwReadLen = 0;

	ComLog("SocketRead.Begin: (%d):(%d)\n", iInputLen, m_iComPortTimeOut);

	FD_ZERO((fd_set FAR *) &readfds);
	FD_SET(m_hSocket, (fd_set FAR *) &readfds);
	seltime.tv_sec  = m_iComPortTimeOut / 1000;
	seltime.tv_usec = (m_iComPortTimeOut % 1000) * 1000;

#ifdef _WIN32
#define CONST const
#else
#define CONST
#endif
	if ((rc = select(FD_SETSIZE,
		(fd_set FAR *)&readfds, (fd_set FAR *) 0, (fd_set FAR *) 0,
		(CONST struct timeval FAR *)&seltime)) < 0) {

		ComLog("SocketRead.select: failed(%d)\n", WSAGetLastError());
		return FALSE;
	}

	ComLog("SocketRead.select: (%d)\n", rc);

	if (rc == 0) { // No Inputdata nach ComPortTimeOut millis

		return FALSE;
	}

	if ((rc = recv(m_hSocket, pcInputBuf, iInputLen, 0)) == SOCKET_ERROR) {

		ComLog("SocketRead.recv: failed(%d)\n", WSAGetLastError());
		return FALSE;
	}

	*pdwReadLen = rc;

	ComLog("SocketRead.End\n");
	
	return TRUE;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
int CChannel::SocketWrite(const char *pcOutputBuf, int iOutputLen, unsigned long *pdwWriteLen)
{
	int rc;

	ComLog("SocketWrite.Begin: (%d)\n", iOutputLen);
	
	if ((rc = send(m_hSocket, pcOutputBuf, iOutputLen, 0)) == SOCKET_ERROR) {

		ComLog("SocketWrite.send: failed(%d)\n", WSAGetLastError());
		return FALSE;
	}

	*pdwWriteLen = rc;

	ComLog("SocketWrite.End\n");
	
	return TRUE;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
int CChannel::OpenSerial(char *szComPort, int iBaudRate, int iTimeOut)
{
	int rc = FALSE;

	if (m_ChannelType == Undef && m_hCom == INVALID_HANDLE_VALUE) {

		if ((rc = SerialOpen(szComPort, iBaudRate, iTimeOut)) == TRUE && m_hCom != INVALID_HANDLE_VALUE) {

			m_ChannelType = Serial;
			m_bOwnHandle  = true;
		}
	}

	return rc;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
int CChannel::OpenSocket(char *szRemoteIP, int iPortNum, int iTimeOut)
{
	int rc = FALSE;

	if (m_ChannelType == Undef && m_hSocket == INVALID_SOCKET) {

		if ((rc = SocketOpen(szRemoteIP, iPortNum, iTimeOut)) == TRUE && m_hSocket != INVALID_SOCKET) {

			m_ChannelType = Socket;
			m_bOwnHandle  = true;
		}
	}

	return rc;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
int CChannel::OpenClass(int iAddress, int iType, char *szComString, int iComValue, int iComTimeOut)
{
	int res = FALSE;

	if (m_ChannelType == Undef) {

		// Je nach Typ der Verbindung die Klasse initialisieren.
		switch (iType) {
		case T_S2_RS232:
		case T_S2_BIM_DIREKT:

			strcpy((char*)m_Com.ComPort, szComString);
			m_Com.Baud   = iComValue;

			strcpy(m_Com.RemoteIP, "");
			m_Com.Socket = INVALID_SOCKET;
			m_Com.PortNo = 0;

			m_Com.AdAbsender = iAddress;
			m_Com.Typ        = iType;
			m_Com.ComTimeOut = iComTimeOut;

			m_iComPortTimeOut = iComTimeOut;
			m_ChannelType = Class;
			m_bOwnHandle  = true;

			// Schnittstelle oeffnen
			res = m_Com.ComOpen();
			break;

		case T_S2_TCP_IP:

			strcpy((char*)m_Com.ComPort, "");
			m_Com.hcom   = INVALID_HANDLE_VALUE;
			m_Com.Baud   = 0;

			strcpy(m_Com.RemoteIP, szComString);
			m_Com.PortNo = iComValue;

			m_Com.AdAbsender = iAddress;
			m_Com.Typ        = iType;
			m_Com.ComTimeOut = iComTimeOut;

			m_iComPortTimeOut = iComTimeOut;
			m_ChannelType = Class;
			m_bOwnHandle  = true;

			// Schnittstelle oeffnen
			res = m_Com.ComOpen();
			break;
		}

		// Nach Misserfolg die Grundeinstellung wiederherstellen
		if (!res) {

			m_ChannelType = Undef;
			m_bOwnHandle  = false;
			m_Com.SetDefault();
			m_Com.SetLogFileName((const char*)m_szLogFileName);
		}
	} 

	return res;
}

// ##################################################################
/**
 * @brief Ermitteln ob die Schnittstelle geoeffnet ist.
 */
int CChannel::IsOpen()
{
	if (((m_ChannelType == Serial) && (m_hCom != INVALID_HANDLE_VALUE)) ||
		((m_ChannelType == Socket) && (m_hSocket != INVALID_SOCKET)) || 
		((m_ChannelType == Class)  && (m_Com.hcom != INVALID_HANDLE_VALUE)) || 
		((m_ChannelType == Class)  && (m_Com.Socket != INVALID_SOCKET)) ) {

		return 1;
	} 

	return 0;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel::Close(void)
{
	if (m_ChannelType == Socket && m_hSocket != INVALID_SOCKET) {
		SocketClose();
	} else if (m_ChannelType == Serial && m_hCom != INVALID_HANDLE_VALUE) {
		SerialClose();
	} else if (m_ChannelType == Class) {

		m_Com.ComClose();
	}

	m_ChannelType = Undef;
	m_bOwnHandle  = false;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
int CChannel::Read(void *pBuf, int iLength, unsigned long *pdwReadLen)
{
	int rc = FALSE;

	char *DataBuf = (char *) pBuf;
	int   DataPos = 0;
	unsigned long DataLen = 0;

	if (m_ChannelType == Socket && m_hSocket != INVALID_SOCKET) {

		for (*pdwReadLen = 0;;) {

			if ((rc = SocketRead(DataBuf + DataPos, iLength, &DataLen)) != TRUE) {
			
				break;
			}

			// Abbruch der Endlosschleife!
			if (DataLen <= 0) {

				break;
			}

			*pdwReadLen += DataLen;

			ComLog("Read.SocketRead: (%d)(%2.2d)(%2.2d)", rc, iLength, DataLen);
			if (m_bLogIsOn && rc) {

				ComLog("(");
				for (unsigned long i = 0; i < *pdwReadLen; i++) {

					ComLog("%2.2x:", *((unsigned char *) pBuf+i));
				}
				ComLog(")");
			}
			ComLog("\n");
			
			iLength -= DataLen;
			if (iLength <= 0) {

				break;
			}

			DataPos  += DataLen;
		}

	} else if (m_ChannelType == Serial && m_hCom != INVALID_HANDLE_VALUE) {

		for (*pdwReadLen = 0;;) {
			if ((rc = SerialRead(DataBuf + DataPos, iLength, &DataLen)) != TRUE) {
			
				break;
			}

			// Abbruch der Endlosschleife!
			if (DataLen <= 0) {

				break;
			}

			*pdwReadLen += DataLen;

			ComLog("Read.SerialRead: (%10.10ld):(%d)(%2.2d)(%2.2d)", clock(), rc, iLength, DataLen);

			if (m_bLogIsOn && rc) {

				ComLog("(");
				for (unsigned long i = 0; i < *pdwReadLen; i++) {

					ComLog("%2.2x:", *((unsigned char *) pBuf+i));
				}
				ComLog(")");
			}
			ComLog("\n");

			iLength -= DataLen;
			if (iLength <= 0) {

				break;
			}

			DataPos  += DataLen;
		}

	} else if (m_ChannelType == Class) {

		return m_Com.Read(pBuf, iLength, pdwReadLen);
	}

	return rc;
}

// ##################################################################
/**
 * @brief Schreiben der Zeichen je nach Kanaltyp.
 *
 * @param pBuf Zeiger auf Buffer fuer Daten.
 * @param iLength Laenge der zu schreibenden Daten
 * @param pdwWriteLen { Enthaelt nach erfolg die Anzahl geschriebener Zeichen. }
 *
 * @return Ob Ausfuehrung erfolgreich oder gescheitert.
 * @retval 0 Schreiben gescheitert.
 * @retval 1 Schreiben erfolgt.
 */
int CChannel::Write(void *pBuf, int iLength, unsigned long *pdwWriteLen)
{
	int rc = FALSE;

	const char *DataBuf = (const char *) pBuf;

	if ((m_ChannelType == Socket) && (m_hSocket != INVALID_SOCKET)) {
	
		*pdwWriteLen = 0;

		rc = SocketWrite(DataBuf, iLength, pdwWriteLen);

		ComLog("Write.SocketWrite: (%d)(%2.2d)(%2.2d)", rc, iLength, *pdwWriteLen);
		if (m_bLogIsOn && rc) {

			ComLog("(");
			for (unsigned long i = 0; i < *pdwWriteLen; i++) {

				ComLog("%2.2x:", *((unsigned char *) pBuf + i));
			}
			ComLog(")");
		}
		ComLog("\n");

	} else if ((m_ChannelType == Serial) && (m_hCom != INVALID_HANDLE_VALUE)) {

		*pdwWriteLen = 0;

		rc = SerialWrite(DataBuf, iLength, pdwWriteLen);

		ComLog("Write.SerialWrite: (%d)(%2.2d)(%2.2d)", rc, iLength, *pdwWriteLen);
		if (m_bLogIsOn && rc) {

			ComLog("(");
			for (unsigned long i = 0; i < *pdwWriteLen; i++) {

				ComLog("%2.2x:", *((unsigned char *) pBuf + i));
			}
			ComLog(")");
		}
		ComLog("\n");

	} else if (m_ChannelType == Class) {

		return m_Com.Write(pBuf, iLength, pdwWriteLen);
	}

	return rc;
}

