/**
 * @file  DFLog.h
 * @brief Schnittstelle fuer die Klasse CDFLog.
 */
//////////////////////////////////////////////////////////////////////
#ifndef _DFLOG_H_
#define _DFLOG_H_

// Initialisierungsdatei verwalten
#ifdef _WIN32
#include "../Common/DFFileIni.h"
#else
#include "extern/DFFileIni.h"
#endif

/* Die Logklasse ist fuer ein standardmaessiges logen von Funktionsaufrufen,
 * so dass Fehler in der Verarbeitung auch noch nach einigen Tagen 
 * nachfolzogen werden koennten.
 * Sie ist ueber eine Initialisierungsdatei mit grundinformationen versorgbar.
 *
 */
#define SUB_FOLDER           "DFCom"

#define INI_FILENAME         "DFCom.ini"
// DFComJJMMTT.log
#define LOG_FILENAME         "%s%sDFCom%02i%02i%02i.log"
#define LOG_SEARCH_FILENAME  "%s%sDFCom*.log"
#define LOG_DELETE_FILENAME  "%s%s%s"

#ifdef _WIN32
#define PATH_BASE            ".\\"
#else
#define PATH_BASE            "./"
#endif

#ifdef _WIN32
#define fullpath             _fullpath
#define mkdir                _mkdir
#define stat                 _stat
#define finddata_t           _finddata_t
#define findfirst            _findfirst
#define findnext             _findnext
#define findclose            _findclose
#endif

class DFLog
{
	bool m_bEnable;                  // Log aktiv?
	char m_szBasePath[MAX_PATH];     // Basispfad fuer Log
	char m_szSubFolder[80];          // Unterordner fuer Logdateien
	char m_szFileName[MAX_PATH];     // Dateiname der Logdatei
	bool m_bEraseFiles;              // Alte Dateien loeschen
	bool m_bElapse;                  // Ausfuehrungszeit angeben

public:
	DFLog();
	virtual ~DFLog();

	// Einlesen der Initialisierungsdatei
	void IniRead();
	// Pruefen auf alte Dateien.
	void Check();

	// Schreiben einer Logzeile, wenn Log aktiv.
	void Begin(const char *Format, ...); // Mit Uhrzeit
	void Append(const char *szFormat, ...);
	void End(const char *szFormat, ...); // Mit Zeilenumbruch

protected:
	CDFFileIni m_fileIni;
	// Start des Logvorganges
	clock_t m_clockBegin;
};

#endif

