/**
 * @file  IntelHex.cpp
 * @brief Implementierung der Klasse CIntelHex.
 */
//////////////////////////////////////////////////////////////////////

#include "../StdAfx.h"
#include "IntelHex.h"

#include <ctype.h>
#include <string.h>
#include <stdio.h>

CIntelHex::CIntelHex()
{

}

CIntelHex::~CIntelHex()
{

}

// ########################################################
// HELPER

UINT16 CIntelHex::HexToUINT16(INT8 *data)
{
	INT32 i;
	UINT16 val = 0;

	for (i=0; i < 4; i++) {

		val = (val << 4);

		if (data[i] >= '0' && data[i] <= '9') {

			val += data[i] - '0';

		} else if (data[i] >= 'a' && data[i] <='f') {

			val += data[i] - 'a' + 10;

		} else {

			val += data[i] - 'A' + 10;
		}
	}

	return val;
}

VOID CIntelHex::UINT16ToHex(UINT16 val, INT8 *data)
{
	INT32 i;

	for (i=3; i >= 0; i--) {

		data[i] = val & 0xF;

		if (data[i] > 9) {
			
			data[i] += 'A' - 10; 
		} else {
		
			data[i] += '0';
		}

		val >>= 4;
	}	
}

UINT8 CIntelHex::HexToUINT8(INT8 *data)
{
	INT32 i;
	UINT8 val = 0;

	for (i=0; i < 2; i++) {

		val = (val << 4);

		if (data[i] >= '0' && data[i] <= '9') {

			val += data[i] - '0';

		} else if (data[i] >= 'a' && data[i] <='f') {

			val += data[i] - 'a' + 10;

		} else {

			val += data[i] - 'A' + 10;
		}
	}

	return val;
}

void CIntelHex::UINT8ToHex(UINT8 val, INT8 *data)
{
	data[0] = (val >> 4) & 0xF;

	if (data[0] > 9) {
		
		data[0] += 'A' - 10; 
	} else {
		
		data[0] += '0';
	}

	data[1] = val & 0xF;

	if (data[1] > 9) {
		
		data[1] += 'A' - 10;
	} else {
		
		data[1] += '0';
	}
}

INT32 CIntelHex::MakeCrc(char *data)
{
	INT32 length = 0, lastByte, check = 0;

	if (data[length] != INTELHEX_START_SIGN) {

		return FALSE;
	}
	length++;

	// Laenge der Datenzeile ermitteln
	while(isxdigit((INT32)data[length])) {
		
		length++;

		if (!(length % 2)) {

			lastByte = HexToUINT8(&data[length - 1]);
			check += lastByte;
		}
	}

	if (length < 9) { // Mindestens 9 Zeichen: ":00000001"
		
		return FALSE;
	}

	// Pruefsumme eintragen
	UINT8ToHex((UINT8)(~(((INT8*)&check)[0] - 1) & 0xFF), &data[length]);

	return TRUE;
}

INT32 CIntelHex::CheckCrc(INT8 *data)
{
	INT32 length = 0, lastByte, check = 0;

	if (data[length] != INTELHEX_START_SIGN) {

		return FALSE;
	}
	length++;

	// Laenge der Datenzeile ermitteln
	while(isxdigit((INT32)data[length])) {
		
		length++;

		if (!(length % 2)) {

			lastByte = HexToUINT8(&data[length - 1]);
			check += lastByte;
		}
	}

	if (length < 11) { // Mindestens 11 Zeichen: ":00000001FF"
		
		return FALSE;
	}

	// Pruefsumme aus Gesamtsumme entfernen
	check -= lastByte;
	// Zweierkomplement
	check = ~(((INT8*)&check)[0] - 1) & 0xFF;
	
	return (check == lastByte);
}

INT32 CIntelHex::MakeTypedDataLine(UINT16 address, UINT8 type, UINT8 *buf, UINT8 length, INT8 *data)
{
	data[INTELHEX_START] = INTELHEX_START_SIGN;    // start of record
	UINT8ToHex(length, &data[INTELHEX_COUNT]);     // byte count
	UINT16ToHex(address, &data[INTELHEX_ADDRESS]); // load address
	UINT8ToHex(type, &data[INTELHEX_TYPE]);        // record type

	// insert data
	UINT8 i;
	for (i=0; i < length; i++) {

		UINT8ToHex(buf[i], &data[INTELHEX_DATA + (i * 2)]);
	}

	// Endezeichen fuer CRC - Laengenermittlung
	data[INTELHEX_DATA + (i * 2)] = 0;

	// checksum suffix
	if (MakeCrc(data)) {

		i++;

		// Nullterminie
		data[INTELHEX_DATA + (i * 2)] = 0;
		return TRUE;
	}

	return FALSE;
}

INT32 CIntelHex::MakeDataLine(UINT16 address, UINT8 *buf, UINT8 length, INT8 *data)
{
	return MakeTypedDataLine(address, INTELHEX_TYPE_DAT, buf, length, data);
}

INT32 CIntelHex::MakeEndLine(INT8 *data)
{
	data[INTELHEX_START] = INTELHEX_START_SIGN;          // start of record
	UINT8ToHex(0, &data[INTELHEX_COUNT]);                // byte count
	UINT16ToHex(0, &data[INTELHEX_ADDRESS]);             // load address
	UINT8ToHex(INTELHEX_TYPE_EOF, &data[INTELHEX_TYPE]); // record type

	// Endezeichen fuer CRC - Laengenermittlung
	data[INTELHEX_DATA] = 0;

	// checksum suffix
	if (MakeCrc(data)) {

		// Nullterminie
		data[INTELHEX_DATA + 2] = 0;
		return TRUE;
	}

	return FALSE;
}

// ########################################################
// Pruefen der Datei mit Inhalt.
INT32 CIntelHex::CheckFile(INT8 *fileName)
{
	UINT32 pos = 0, numRecord = 0;
	INT8 buf[530];
	UINT8 fail = FALSE;

	FILE *fp = fopen(fileName, "r");

	if (fp == NULL) {
	
		return FALSE;
	}

	memset(buf, 0, sizeof(buf));
	while(!feof(fp)) {
		
		fread(&buf[pos], 1, 1, fp);

		if (buf[pos] == INTELHEX_START_SIGN && numRecord) {
		
			if (!CheckCrc(buf)) {

				fail = TRUE;
				break;
			}

			memset(buf, 0, sizeof(buf));

			pos = 0;
			numRecord++; // Naechster Datensatz in Vorbereitung
			buf[0] = INTELHEX_START_SIGN;
		}

		if (pos >= sizeof(buf)) {

			fail = TRUE;
			break;
		}
		// Erster Datensatz angefangen
		if (!numRecord) {
		
			numRecord++;
		}

		pos++;
	}

	fclose(fp);

	if (!fail) {

		if (!CheckCrc(buf)) {

			fail = TRUE;
		} else {

			// Abschlusszeile pruefen (End-of-file record) 
			if (GetType(buf) != INTELHEX_TYPE_EOF) {

				fail = TRUE;
			}
		}
	}

	return !fail;
}

UINT8 CIntelHex::GetCount(INT8 *data)
{
	return HexToUINT8(&data[INTELHEX_COUNT]);
}

UINT16 CIntelHex::GetAddress(INT8 *data)
{
	return HexToUINT16(&data[INTELHEX_ADDRESS]);
}

UINT8 CIntelHex::GetType(INT8 *data)
{
	return HexToUINT8(&data[INTELHEX_TYPE]);
}

UINT8 CIntelHex::GetData(INT8 *data, UINT8 *dest)
{// vom Datensatztyp unabhaengig
	UINT16 num, length = GetCount(data);
	if (length > 0xFF) {
	
		return 0;
	}

	for (num=0; num < length; num++) {
	
		dest[num] = HexToUINT8(&data[INTELHEX_DATA + 2 * num]);
	}

	return (UINT8)num;
}

UINT8 CIntelHex::GetData8(INT8 *data, UINT8 num)
{
	if (num > GetCount(data)) {
	
		return 0;
	}

	return HexToUINT8(&data[INTELHEX_DATA + 2 * num]);
}

UINT16 CIntelHex::GetData16(INT8 *data, UINT8 num)
{
	if ((num + 1) > GetCount(data)) {
	
		return 0;
	}

	return HexToUINT16(&data[INTELHEX_DATA + 2 * num]);
}

