/**
 * @file  AE3Com.cpp
 * @brief Definitionen der Exportroutinen.
 */
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "AE3ComProject.h"
#include "AE3Com.h"

// ########################################################
// #####################   EXPORTE    #####################
// ########################################################

#ifdef _WIN32
// ########################################################
/** 
 * @fn    CALLBACK DllGetVersion(DLLVERSIONINFO *pdvi)
 * @brief Ermitteln der DLL - Version. Kompatible zu Windows - Standard.
 *  
 * @param pdvi       { Zeiger auf Struktur zur Informationsuebergabe. }
 */
extern "C" HRESULT PASCAL EXPORT CALLBACK DllGetVersion(DLLVERSIONINFO *pdvi)
{
	char szModuleName[MAX_PATH];
	DWORD dwHandle;
	DWORD dwLen;
	LPVOID lpBuffer = NULL;
	UINT uiLen;

	unsigned char *pucData = NULL;

	// zulaessige Bereiche pruefen
	if (pdvi == NULL) {

		return E_ABORT;
	}

	if (pdvi->cbSize != sizeof(DLLVERSIONINFO)) {
	
		return E_ABORT;
	}
	// Modulname der DLL
	if (!GetModuleFileName(theApp.m_hInstance, szModuleName, MAX_PATH)) {
	
		return E_ABORT;
	}
	// Laenge der Version - Resource
	dwLen = GetFileVersionInfoSize(szModuleName, &dwHandle);
	if (!dwLen) {
	
		return E_ABORT;
	}

	pucData = new unsigned char[dwLen + 100]; // unicode
	// DLL - Versioninfo ermitteln
	if (!GetFileVersionInfo(szModuleName, dwHandle, dwLen, (LPVOID)pucData)) {
	
		delete pucData;
		return E_ABORT;
	}
	// Pointer auf allgemeine Infostruktur ermitteln
	if (!VerQueryValue((LPVOID)pucData, "\\\0", &lpBuffer, &uiLen)) {

		delete pucData;
		return E_ABORT;
	}
	// Daten zurueckgeben
	VS_FIXEDFILEINFO *pvs_ffi = (VS_FIXEDFILEINFO*)lpBuffer;
	pdvi->dwMajorVersion = pvs_ffi->dwFileVersionMS >> 16;
	pdvi->dwMinorVersion = pvs_ffi->dwFileVersionMS & 0x0000FFFF;
	pdvi->dwBuildNumber  = pvs_ffi->dwFileVersionLS >> 16;
	pdvi->dwPlatformID   = DLLVER_PLATFORM_WINDOWS;

	delete pucData;

	return NOERROR;
}
#endif

#undef FUNC_NAME
#define FUNC_NAME "DFCGetErrorText"
// ########################################################
extern "C" __declspec(dllexport)
 
void __stdcall DFCGetErrorText(int iChannelNum, int iError, int iLanguage, char *szText, int iLength)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iError, iLanguage);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		(szText == NULL) ) {

		theApp.m_log.End("\t\t%i", iLength);
		return;
	}

	switch(iError) {
	case ERROR_CRC		:
		snprintf(szText, iLength, "CRC Fehler.");
		break;
	case ERROR_COM_INIT	:
		snprintf(szText, iLength, "Oeffnen der Schnittstelle fehlgeschlagen.");
		break;
	case ERROR_COM_CLOSED	:
		snprintf(szText, iLength, "Schnittstelle ist geschlossen.");
		break;
	case ERROR_COM		:
		snprintf(szText, iLength, "Allgemeiner Fehler waehrend Kommunikation.");
		break;
	case ERROR_NO_ANSWER 	:
		snprintf(szText, iLength, "Geraet antwortet nicht.");
		break;
	case ERROR_AT_SENDING	:
		snprintf(szText, iLength, "Fehler waehrend Senden eingetreten.");
		break;
	case ERROR_AT_RECIEVING	:
		snprintf(szText, iLength, "Fehler waehrend Empfang eingetreten.");
		break;
	case ERROR_SENDER	:
		snprintf(szText, iLength, "Antwort von falschem Geraet oder Antwort nicht sinnvoll.");
		break;
	case ERROR_BUS_NUM	:
		snprintf(szText, iLength, "Keine gueltige Busnummer. (1 - 31, 254)");
		break;
	case ERROR_CHANNEL	:
		snprintf(szText, iLength, "Kanal nicht vorhanden oder nicht erstellbar.");
		break;
	case ERROR_OPEN		:
		snprintf(szText, iLength, "Oeffnen einer Datei gescheitert.");
		break;
	case ERROR_FORMAT	:
		snprintf(szText, iLength, "z.B. falsches Format einer Setup-Datei.");
		break;
	case ERROR_LIST		:
		snprintf(szText, iLength, "Fehler bei Listendatenverarbeitung.");
		break;
	case ERROR_FW_VERSION	:
		snprintf(szText, iLength, "Firmwareversion nicht kompatible zu Routine.");
		break;
	case ERROR_IS_DATA	:
		snprintf(szText, iLength, "Es sind noch Daten im Geraet vorhanden.");
		break;
	case ERROR_BREAK	:
		snprintf(szText, iLength, "Durchfuehrung wurde von Anwender abgebrochen.");
		break;
	case ERROR_NO_RD	:
		snprintf(szText, iLength, "Keine Datensatzbeschreibung geladen.");
		break;
	case ERROR_NO_LD	:
		snprintf(szText, iLength, "Keine Listenbeschreibung geladen.");
		break;
	case ERROR_FLASH_PROG:
		snprintf(szText, iLength, "Fehler beim Programmieren des Flash, Firmware.");
		break;
	case ERROR_RANGE:
		snprintf(szText, iLength, "Parameter einer Funktion ausserhalb ihres Gueltigkeitsbereiches.");
		break;
	case ERROR_NOT_DEFINED:
		snprintf(szText, iLength, "Globale Variable nicht definiert.");
		break;
	case ERROR_NO_DATA:
		snprintf(szText, iLength, "Anfrage obwohl keine Daten.");
		break;
	case ERROR_MULTI:
		snprintf(szText, iLength, "Dublikat von Datensaetzen abgefangen.");
		break;
	case ERROR_BOOTVERSION:
		snprintf(szText, iLength, "Geraetedatei nicht mit Bootloaderversion kompatible.");
		break;
	case ERROR_BOOTFILEVERSION:
		snprintf(szText, iLength, "Firmwaredatei nicht mit Bootloaderversion kompatible.");
		break;
	case ERROR_VAR_VALUE:
		snprintf(szText, iLength, "Wert einer Variablen konnte nicht gesetzt werden.");
		break;
	case ERROR_FUNCTION_UNAVAILABLE:
		snprintf(szText, iLength, "Funktion steht im vorliegenden Fall nicht zur Verfuegung.");
		break;
	default:
		snprintf(szText, iLength, "Unbekannter Fehler.");
		break;
	}

	theApp.m_log.End("\t%s\t%i", szText, iLength);
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSetLogFileName"
// ########################################################
extern "C" __declspec(dllexport)

void __stdcall DFCSetLogFileName(int iChannelNum, const unsigned char *szLogFileName)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t%s", FUNC_NAME, iChannelNum, 
		(szLogFileName == NULL) ? "" : (char*)szLogFileName);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		(szLogFileName == NULL) ) {

		theApp.m_log.End("");
		return;
	}

	CAE3Channel *pChannel = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channel vorhanden
	if (pChannel != NULL) {

		pChannel->SetLogFileName((const char*)szLogFileName);
	}

	theApp.m_log.End("");
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSetLogOn"
// ########################################################
extern "C" __declspec(dllexport)

void __stdcall DFCSetLogOn(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i", FUNC_NAME, iChannelNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ) {

		theApp.m_log.End("");
		return;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channel vorhanden
	if (pChannel != NULL) {

		pChannel->SetLogOn();
	}

	theApp.m_log.End("");
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSetLogOff"
// ########################################################
extern "C" __declspec(dllexport)

void __stdcall DFCSetLogOff(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i", FUNC_NAME, iChannelNum);
	
	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ) {

		theApp.m_log.End("");
		return;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channel vorhanden
	if (pChannel != NULL) {

		pChannel->SetLogOff();
	}

	theApp.m_log.End("");
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetComPort"
// ########################################################
extern "C" __declspec(dllexport)

HANDLE __stdcall DFCGetComPort(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i", FUNC_NAME, iChannelNum);
	
	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ) {

		theApp.m_log.End("\t(%i)", INVALID_HANDLE_VALUE);
		return INVALID_HANDLE_VALUE;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channel vorhanden
	if (pChannel != NULL) {

		HANDLE hRes = pChannel->GetComPort();
		theApp.m_log.End("\t(%i)", hRes);
		return hRes;
	}

	theApp.m_log.End("\t(%i)", INVALID_HANDLE_VALUE);
	return INVALID_HANDLE_VALUE;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSetComPort"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCSetComPort(int iChannelNum, HANDLE hComPort, int iType, int iComValue, int iTimeOut)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i", FUNC_NAME, iChannelNum);
	
	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ) {

		theApp.m_log.End("\t%i\t%i\t%i\t%i\t(0)", hComPort, iType, iComValue, iTimeOut);
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channel vorhanden
	if (pChannel != NULL) {

		int iRes = pChannel->SetComPort(hComPort, iType, iComValue, iTimeOut);
		theApp.m_log.End("\t%i\t%i\t%i\t%i\t(%i)", hComPort, iType, iComValue, iTimeOut, iRes);
		return iRes;
	}

	theApp.m_log.End("\t%i\t%i\t%i\t%i\t(0)", hComPort, iType, iComValue, iTimeOut);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComOpenSerial"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCComOpenSerial(int iChannelNum, char *szComPort, int iBaudRate, int iTimeOut)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t%s\t%i\t%i", FUNC_NAME, iChannelNum, 
		(szComPort == NULL) ? "" : szComPort, iBaudRate, iTimeOut);
	
	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		(szComPort == NULL) ) {

		theApp.m_log.End("\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channel vorhanden
	if (pChannel != NULL) {

		int iRes = pChannel->OpenSerial(szComPort, iBaudRate, iTimeOut);
		theApp.m_log.End("\t(%i)", iRes);
		return iRes;
	}

	theApp.m_log.End("\t(0)");
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComOpenSocket"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCComOpenSocket(int iChannelNum, char *szRemoteIP, int iPortNum, int iTimeOut)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%s\t%i\t%i", FUNC_NAME, iChannelNum, 
		(szRemoteIP == NULL) ? "" : szRemoteIP, iPortNum, iTimeOut);
	
	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		(szRemoteIP == NULL) ) {

		theApp.m_log.End("\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channel vorhanden
	if (pChannel != NULL) {

		int iRes = pChannel->OpenSocket(szRemoteIP, iPortNum, iTimeOut);
		theApp.m_log.End("\t(%i)", iRes);
		return iRes;
	}

	theApp.m_log.End("\t(0)");
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComOpenIV"
// ########################################################
// Multimaster

extern "C" __declspec(dllexport)

int __stdcall DFCComOpenIV(int iChannelNum, int iBusNum, int iType, char *szComString, int iComValue, int iComTimeOut)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t%i\t%i\t%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum, iType, 
		(szComString == NULL) ? "" : szComString, iComValue, iComTimeOut);
	
	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(szComString == NULL) ) {

		theApp.m_log.End("\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channel vorhanden
	if (pChannel != NULL) {

		int iRes = pChannel->OpenClass(iBusNum, iType, szComString, iComValue, iComTimeOut);
		theApp.m_log.End("\t(%i)", iRes);
		return iRes;
	}

	theApp.m_log.End("\t(0)");
	return 0;
}

// ________________________________________________________

#undef FUNC_NAME
#define FUNC_NAME "DFCComClose"
// ########################################################
extern "C" __declspec(dllexport)

void   __stdcall DFCComClose(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i", FUNC_NAME, iChannelNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ) {

		theApp.m_log.End("");
		return;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channel vorhanden
	if (pChannel != NULL) {

		pChannel->Close();
	}

	theApp.m_log.End("");
}

#undef FUNC_NAME
#define FUNC_NAME "DFCCheckDevice"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCCheckDevice(int iChannelNum, int iBusNum, int *piError, int iDevicePollRetry)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t%i\t%i\t(0)", ERROR_RANGE, iDevicePollRetry);
		return 0;
	}
	*piError = ERROR_NO;

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	if (pChannel != NULL) {

		int iRes;

		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t%i\t%i\t(0)", *piError, iDevicePollRetry);
			return 0;
		}

		iRes = pChannel->CheckDevice(iBusNum, piError, iDevicePollRetry);
		theApp.m_log.End("\t%i\t%i\t(%i)", *piError, iDevicePollRetry, iRes);

		return iRes;

	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t%i\t%i\t(0)", *piError, iDevicePollRetry);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetDevicePollRetry"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCGetDevicePollRetry(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i", FUNC_NAME, iChannelNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ) {

		theApp.m_log.End("\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	if (pChannel != NULL) {

		theApp.m_log.End("\t(%i)", pChannel->m_iDevicePollRetry);
		return pChannel->m_iDevicePollRetry;
	}

	theApp.m_log.End("\t(0)");
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComCollectData"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCComCollectData(int iChannelNum, int iBusNum, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t%i\t(%i)", ERROR_RANGE, -1);
		return -1;
	}
	*piError = ERROR_NO;

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes;
		
		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t%i\t(%i)", *piError, -1);
			return -1;
		}

		// Haelt Geraeteversion fuer Datensatzrueckgabe
		pChannel->m_usLastCollectVersion = 0;

		// Wechsel der Busnummer und somit loeschen des Datenpuffer
		iRes = pChannel->ChangeBusNum(iBusNum);
		if (!iRes) {
		
			*piError = ERROR_BUS_NUM;
			theApp.m_log.End("\t%i\t(%i)", *piError, -1);
			return -1;
		}
		
		// Pruefen der Firmwareversion
		if (!(iRes = pChannel->CheckComVersion(pChannel->GetBusNum(), piError))) {
		
			//*piError = ERROR_FW_VERSION;
			theApp.m_log.End("\t%i\t(%i)", *piError, -1);
			return -1;
		}

		// Halten der FWVersion fuer ComGetDatensatz
		pChannel->m_usLastCollectVersion = iRes;

		switch (iRes) {
		case 0x0200:
		case 0x0201:
		case 0x0300: 
		case 0x0400:
		case 0x0401:
		case 0x0402:
		case 0x0500: // PZE - Master IV
		case 0x0501:
		case 0x0502:

			// Datensatz ermitteln
			iRes = pChannel->ReadRecord(pChannel->GetBusNum(), pChannel->m_pucRecordBuf, 
				&pChannel->m_iRecordLength, &pChannel->m_iRecordError);
			
			int i;

			switch ( iRes ) {
			case 0: // Keine Datensaetzte vorhanden
				
				for (i=0; i < MAX_MB_PAKET; i++) {
				
					pChannel->m_pucRecordBuf[i] = 0;
				}
				pChannel->m_iRecordLength = 0;
				pChannel->m_iRecordError = 0;

				theApp.m_log.End("\t%i\t(%i)", ERROR_NO, 0);
				return 0;

			case 1: // Online Datensatz
			case 2: // Wiederholter Online Datensatz
	
				theApp.m_log.End("\t%i\t(%i)", ERROR_NO, 2);
				return 2;

			case 3: // Offline Datensatz
			case 4: // Wiederholter Offline Datensatz

				theApp.m_log.End("\t%i\t(%i)", ERROR_NO, 1);
				return 1;
			}

			// Fehler bei Ermittlung
			*piError = pChannel->m_iRecordError;
			pChannel->ClearCom();
			break;

#ifdef _WIN32
		case 0x0103: // Alte Firmware mit Speicher auslesen.
		case 0x0102:
			
			iRes = pChannel->OldCollectData(iBusNum, piError);
			theApp.m_log.End("\t%i\t(%i)", *piError, iRes);
			return iRes;
#endif

		default:
			*piError = ERROR_FW_VERSION;
		}

		theApp.m_log.End("\t%i\t(%i)", *piError, -1);
		return -1;
	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t%i\t(%i)", *piError, -1);
	return -1;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComGetDatensatz"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCComGetDatensatz(int iChannelNum, unsigned char *pucBuf, int *piLength, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i", FUNC_NAME, iChannelNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		(pucBuf == NULL) || (piLength == NULL) || (piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t\t\t%i\t(%i)", ERROR_RANGE, -1);
		return -1;
	}
	*piError = ERROR_NO;

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes = -1;

		// Wenn FWVersion des letzten CollectData 0 ist dann wurden die
		// Daten nicht erfolgreich abgeholt und es sind keine vorhanden.
		if (!pChannel->m_usLastCollectVersion) {
		
			theApp.m_log.End("\t\t\t%i\t(%i)", ERROR_NO, 0);
			return 0;
		}

		switch(pChannel->m_usLastCollectVersion) {
		case 0x0200:
		case 0x0201:
		case 0x0300:
		case 0x0400:
		case 0x0401:
		case 0x0402:
		case 0x0500: // PZE - Master IV
		case 0x0501:
		case 0x0502:

			// Wenn BusNum des letzten Datensatzpakets 0 ist dann sind 
			// keine gueltigen Daten vorhanden.
			if (pChannel->m_pucRecordBuf[0] == 0) {

				theApp.m_log.End("\t\t\t%i\t(%i)", ERROR_NO, 0);
				return 0;
			}

			// Daten in Puffer uebergeben
			unsigned char ucTmp[MAX_MB_PAKET_DATA * 3 + 1];
			int i, iFail;
			// Zu groe Datensaetze abfangen!
			if ((pChannel->m_iRecordLength - 6) > MAX_MB_PAKET_DATA) {
				pChannel->m_iRecordLength -= (pChannel->m_iRecordLength - 6) - MAX_MB_PAKET_DATA;
			}
			for (i=0; i < (pChannel->m_iRecordLength - 6); i++) {
			
				pucBuf[i] = pChannel->m_pucRecordBuf[i + 4];
				sprintf((char*)&ucTmp[i * 3], "%02X ", pChannel->m_pucRecordBuf[i + 4]);
			}
			ucTmp[i * 3] = 0;

			*piLength = pChannel->m_iRecordLength - 6;

			if ((pChannel->m_usLastCollectVersion == 0x0200) &&
				((pChannel->m_pucRecordBuf[3] == MB_ANSW_OFFLINE_RECORD) || 
				 (pChannel->m_pucRecordBuf[3] == MB_ANSW_OLD_OFFLINE_RECORD)) ) {

				// Fehler bei Offline in CW 0x200: konstant 3 zuviel
				// Fehler bei Offline in CW 0x201 bei FW 2.8: konstant 3 zuviel aber 2.9 nicht!
				*piLength -= 3; 
			}
			theApp.m_log.Append("\t%s\t%i", ucTmp, *piLength);

			// Schnittstelle geoeffnet?
			if (!pChannel->IsOpen()) {

				*piError = ERROR_COM_CLOSED;
				theApp.m_log.End("\t%i\t(%i)", *piError, -1);
				return -1;
			}

			// Datensatz quittieren (3x -> Versuchen)
			for (i=0; (i < 3) && ((iRes = pChannel->QuitRecord(pChannel->GetBusNum(), pChannel->m_pucRecordBuf,
				&pChannel->m_iRecordLength, &pChannel->m_iRecordError)) == -1); i++);

			// Quittierung erfolgreich?
			if (iRes != -1) {
				// Vorherigen Datensatz fuer Duplikattest halten.
				for (i=0; i < MAX_MB_PAKET; i++) {
				
					pChannel->m_pucTmpRecordBuf[i] = pChannel->m_pucRecordBuf[i];
				}
				
				// Naechsten Datensatz mit Duplikattest anfordern
				for (i=0, iFail=0; (i < 5) && (iRes = pChannel->ReadRecord(pChannel->GetBusNum(), pChannel->m_pucRecordBuf, 
					&pChannel->m_iRecordLength, &pChannel->m_iRecordError)); i++) {

					// Duplikattest durchfuehren!
					if (memcmp(pChannel->m_pucRecordBuf, pChannel->m_pucTmpRecordBuf, MAX_MB_PAKET) == 0) {

						// Datensatz wurde quittiert und ein identischer
						// geliefert?!?
						iFail++;
					} else {
					
						break;
					}
				}

				// Was eigentlich nicht moeglich ist, wird hier abgefangen!
				if (iFail == 5) {

					for (i=0; i < MAX_MB_PAKET; i++) {
					
						pChannel->m_pucTmpRecordBuf[i] = 0;
						pChannel->m_pucRecordBuf[i] = 0;
					}
					pChannel->m_iRecordLength = 0;
					pChannel->m_iRecordError = 0;

					*piError = ERROR_MULTI;
					theApp.m_log.End("\t%i\t(%i)", *piError, -1);
					return -1;
				}
			} 
			
			switch ( iRes ) {
			case 0: // Keine Datensaetzte vorhanden
				// Aktuell uebergebener ist letzter moeglicher, bei Quittierungsmisserfolg.
				
				for (i=0; i < MAX_MB_PAKET; i++) {
					
					pChannel->m_pucTmpRecordBuf[i] = 0;
					pChannel->m_pucRecordBuf[i] = 0;
				}
				pChannel->m_iRecordLength = 0;
				pChannel->m_iRecordError = 0;
				
				theApp.m_log.End("\t%i\t(%i)", ERROR_NO, 2);
				return 2;
				
				// Online Datensaetze
			case 1: // Online Datensatz
			case 2: // Wiederholter Online Datensatz
				
				theApp.m_log.End("\t%i\t(%i)", ERROR_NO, 1);
				return 1;
				
				// Offline Datensaetze
			case 3: // Offline Datensatz
			case 4: // Wiederholter Offline Datensatz
				
				theApp.m_log.End("\t%i\t(%i)", ERROR_NO, 1);
				return 1; 
				
			}

			// Fehler bei Ermittlung
			*piError = pChannel->m_iRecordError;
			pChannel->ClearCom();
			break;

#ifdef _WIN32
		case 0x0102:
		case 0x0103:

			iRes = pChannel->GetDatensatz(pucBuf, piLength, piError);
			// Je nach Ergebnis Log - ausfuehren
			switch(iRes) {
			case 0:
				theApp.m_log.End("\t\t\t%i\t(%i)", ERROR_NO, iRes);
				break;
			case 1:
			case 2: {
				unsigned char ucTmp[MAX_MB_PAKET_DATA * 3 + 1];
				int i;
				for (i=0; i < *piLength; i++) {
				
					sprintf((char*)&ucTmp[i * 3], "%02X ", pucBuf[i]);
				}
				ucTmp[i * 3] = 0;
				theApp.m_log.End("\t%s\t%i\t%i\t(%i)", ucTmp, *piLength, ERROR_NO, iRes);
				break;}
			default:
				theApp.m_log.End("\t\t\t%i\t(%i)", *piError, iRes);
			}

			return iRes;
#endif

		default:
			*piError = ERROR_FW_VERSION;
		}

		theApp.m_log.End("\t%i\t(%i)", *piError, -1);
		return -1;

	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t%i\t(%i)", *piError, -1);
	return -1;
}


#undef FUNC_NAME
#define FUNC_NAME "DFCReadRecord"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCReadRecord(int channelID, int deviceID, char *buf, int *length, int *error)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAE3Channel *pChannel = theApp.GetChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (error) *error = ERROR_CHANNEL;
		return -1;
	}

	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if ((buf == NULL) || (length == NULL) || (error == NULL) ) {

		if (error) *error = ERROR_RANGE;
		theApp.m_log.End("\t\t\t%i\t(%i)", ERROR_RANGE, -1);
		return -1;
	}
	*error = ERROR_NO;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*error = ERROR_COM_CLOSED;
		theApp.m_log.End("\t\t\t%i\t(%i)", *error, -1);
		return -1;
	}

	int res = pChannel->DFReadRecord(deviceID, buf, length, error);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (res == -1) {
		pChannel->ResetWithError(*error);

		theApp.m_log.Append("\t\t");
	} else {

		// Daten fuer Log in Puffer uebergeben
		unsigned char ucTmp[MAX_MB_PAKET_DATA * 3 + 1];
		int i;
		for (i=0; i < *length; i++) {
		
			sprintf((char*)&ucTmp[i * 3], "%02X ", buf[i]);
		}
		ucTmp[i * 3] = 0;

		theApp.m_log.Append("\t%s\t%i", ucTmp, *length);
	}

	theApp.m_log.End("\t%i\t(%i)", *error, res);
	return res;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCQuitRecord"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCQuitRecord(int channelID, int deviceID, int *error)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CAE3Channel *pChannel = theApp.GetChannel(channelID);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL) {

		if (error) *error = ERROR_CHANNEL;
		return -1;
	}

	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, channelID, deviceID);

	// zulaessige Bereiche pruefen
	if (error == NULL ) {

		if (error) *error = ERROR_RANGE;
		theApp.m_log.End("\t%i\t(%i)", ERROR_RANGE, -1);
		return -1;
	}
	*error = ERROR_NO;

	// Schnittstelle geoeffnet?
	if (!pChannel->IsOpen()) {

		*error = ERROR_COM_CLOSED;
		theApp.m_log.End("\t%i\t(%i)", *error, -1);
		return -1;
	}

	int res = pChannel->DFQuitRecord(deviceID, error);
	// Nach Kommunikationsfehler, Kanal bereinigen
	if (res == -1) pChannel->ResetWithError(*error);

	theApp.m_log.End("\t%i\t(%i)", *error, res);
	return res;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComSendMessage"
// ########################################################
extern "C" __declspec(dllexport)
 
int __stdcall DFCComSendMessage(int iChannelNum, int iBusNum, 
		unsigned char ucDelayTime, unsigned char ucAction, 
		unsigned char ucAudio, char *szMessage, int iLength)
{
	AFX_MANAGE_STATE(AfxGetAfxStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i\t%i\t%i\t%i\t%s\t%i", FUNC_NAME, 
		iChannelNum, iBusNum, ucDelayTime, ucAction, ucAudio, 
		(szMessage == NULL) ? "" : szMessage, iLength);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		!((iLength >= 0) && (iLength <= 242)) ||
		(szMessage == NULL) ) {

		theApp.m_log.End("\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes, iError = ERROR_NO;

		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			theApp.m_log.End("\t(0)");
			return 0;
		}

		iRes = pChannel->SendMessage(iBusNum, ucDelayTime, ucAction, ucAudio, 
			szMessage, iLength, &iError);

		if (!iRes && (iError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t(%i)", iRes);
		return iRes;
	}

	theApp.m_log.End("\t(0)");
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComSendInfotext"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCComSendInfotext(int iChannelNum, int iBusNum, char *szMessage, int iLength)
{
	AFX_MANAGE_STATE(AfxGetAfxStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i\t%s\t%i", FUNC_NAME,
		iChannelNum, iBusNum, (szMessage == NULL) ? "" : szMessage, iLength);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		!((iLength >= 0) && (iLength <= 242)) ||
		(szMessage == NULL) ) {

		theApp.m_log.End("\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes, iError = ERROR_NO;

		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			theApp.m_log.End("\t(0)");
			return 0;
		}

		iRes = pChannel->SendInfotext(iBusNum, szMessage, iLength, &iError);
		
		if (!iRes && (iError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t(%i)", iRes);
		return iRes;
	}

	theApp.m_log.End("\t(0)");
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComClearData"
// ########################################################
extern "C" __declspec(dllexport)
 
int __stdcall DFCComClearData(int iChannelNum, int iBusNum)
{
	AFX_MANAGE_STATE(AfxGetAfxStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ) {

		theApp.m_log.End("\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes, iError = ERROR_NO;
		
		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			theApp.m_log.End("\t(0)");
			return 0;
		}

		pChannel->m_usLastDeviceVersion = 0;
		
		// Pruefen und halten der Firmwareversion
		if (!(iRes = pChannel->CheckComVersion(iBusNum, &iError))) {
		
			//iError = ERROR_FW_VERSION;
			theApp.m_log.End("\t(0)");
			return 0;
		}
		
		pChannel->m_usLastDeviceVersion = iRes;

		iRes = pChannel->ClearData(iBusNum, &iError);

		if (!iRes && (iError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t(%i)", iRes);
		return iRes;
	}

	theApp.m_log.End("\t(0)");
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComSetTime"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCComSetTime(int iChannelNum, int iBusNum, unsigned char *pucDateTime)
{
	AFX_MANAGE_STATE(AfxGetAfxStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ) {

		theApp.m_log.End("\t\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes, iError = ERROR_NO;

		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			theApp.m_log.End("\t\t(0)");
			return 0;
		}

		iRes = pChannel->SetDateTime(iBusNum, pucDateTime, &iError);

		if (!iRes && (iError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t\t(%i)", iRes);
		return iRes;
	}

	theApp.m_log.End("\t\t(0)");
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComGetTime"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCComGetTime(int iChannelNum, int iBusNum, unsigned char *pucDateTime)
{
	AFX_MANAGE_STATE(AfxGetAfxStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(pucDateTime == NULL) ) {

		theApp.m_log.End("\t\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes, iError = ERROR_NO;

		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			theApp.m_log.End("\t\t(0)");
			return 0;
		}

		iRes = pChannel->GetDateTime(iBusNum, pucDateTime, &iError);

		if (!iRes && (iError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		if (iRes) {
			theApp.m_log.End("\t%02i%02i-%02i-%02i %02i:%02i:%02i\t(%i)", 
				pucDateTime[0], pucDateTime[1], pucDateTime[2], pucDateTime[3],
				pucDateTime[4], pucDateTime[5], pucDateTime[6], iRes);
		} else {
			theApp.m_log.End("\t\t(%i)", iRes);
		}

		return iRes;
	}

	theApp.m_log.End("\t\t(0)");
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCMakeListe"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCMakeListe(int iChannelNum, int iListNum, int iLineCount, int iListSize, 
						   unsigned char *pucBuf, int iListMemSizeNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i\t%i\t%i\t\t%i", FUNC_NAME, iChannelNum, 
		iListNum, iLineCount, iListSize, iListMemSizeNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!((iListNum >= 0) && (iListNum < MAX_DATENSATZ_BESCHREIBUNG)) ||
		(pucBuf == NULL) ) {

		theApp.m_log.End("\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes = pChannel->MakeListe(iListNum, iLineCount, iListSize, pucBuf);
		theApp.m_log.End("\t(%i)", iRes);
		return iRes;
	}

	theApp.m_log.End("\t(0)");
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCClrListenBuffer"
// ########################################################
extern "C" __declspec(dllexport) 

void __stdcall DFCClrListenBuffer(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i", FUNC_NAME, iChannelNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ) {

		theApp.m_log.End("");
		return;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		pChannel->ClearListBuffer();
	}

	theApp.m_log.End("");
}

#undef FUNC_NAME
#define FUNC_NAME "DFCLoadListen"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCLoadListen(int iChannelNum, int iBusNum, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t%i\t(%i)", ERROR_RANGE, 0);
		return 0;
	}
	*piError = ERROR_NO;

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes;

		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		pChannel->m_usLastDeviceVersion = 0;
		
		// Pruefen und halten der Firmwareversion
		if (!(iRes = pChannel->CheckComVersion(iBusNum, piError))) {
		
			//iError = ERROR_FW_VERSION;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}
		
		pChannel->m_usLastDeviceVersion = iRes;

		iRes = pChannel->LoadListData(iBusNum, piError);

		if (!iRes && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t%i\t(%i)", *piError, iRes);
		return iRes;

	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t%i\t(%i)", *piError, 0);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCLoadDatensatzbeschreibung"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCLoadDatensatzbeschreibung(int iChannelNum, int iBusNum, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t%i\t(%i)", ERROR_RANGE, 0);
		return 0;
	}
	*piError = ERROR_NO;

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes;

		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		iRes = pChannel->GetTableDescription(iBusNum, FALSE, piError);

		if (!iRes && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t%i\t(%i)", *piError, iRes);
		return iRes;
	} 
	
	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t%i\t(%i)", *piError, 0);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCLoadListenbeschreibung"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCLoadListenbeschreibung(int iChannelNum, int iBusNum, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t%i\t(%i)", ERROR_RANGE, 0);
		return 0;
	}
	*piError = ERROR_NO;

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes;

		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		iRes = pChannel->GetTableDescription(iBusNum, TRUE, piError);

		if (!iRes && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t%i\t(%i)", *piError, iRes);
		return iRes;
	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t%i\t(%i)", *piError, 0);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCDatBCnt"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCDatBCnt(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i", FUNC_NAME, iChannelNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ) {

		theApp.m_log.End("\t(0)");
		return FALSE;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes = pChannel->GetTableCount(FALSE);
		theApp.m_log.End("\t(%i)", iRes);
		return iRes;
	}

	theApp.m_log.End("\t(0)");
	return FALSE;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCListBCnt"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCListBCnt(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i", FUNC_NAME, iChannelNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ) {

		theApp.m_log.End("\t(0)");
		return FALSE;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes = pChannel->GetTableCount(TRUE);
		theApp.m_log.End("\t(%i)", iRes);
		return iRes;
	}

	theApp.m_log.End("\t(0)");
	return FALSE;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCDatBDatensatz"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCDatBDatensatz(int iChannelNum, int iDataNum, unsigned char *szName, int *piFieldCount)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iDataNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!((iDataNum >= 0) && (iDataNum < MAX_DATENSATZ_BESCHREIBUNG)) ||
		(szName == NULL) || (piFieldCount == NULL) ) {

		theApp.m_log.End("\t\t\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes;
		UCHAR szTableName[MAX_DS_NAME];
		UCHAR ucFieldCount;

		iRes = pChannel->GetTableRecord(iDataNum, szTableName, &ucFieldCount);
		
		if (iRes) {

			strcpy((char*)szName, (const char*)szTableName);
			*piFieldCount = ucFieldCount;
		}

		theApp.m_log.End("\t%s\t%i\t(%i)", szName, *piFieldCount, iRes);
		return iRes;
	}

	theApp.m_log.End("\t\t\t(0)");
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCListBDatensatz"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCListBDatensatz(int iChannelNum, int iDataNum, unsigned char *szName, 
								int *piFieldCount, int *piVar)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iDataNum);

	// zulaessige Bereiche pruefen
	if (!( ((iDataNum >= 0) && (iDataNum < MAX_DATENSATZ_BESCHREIBUNG)) ||
		   ((iDataNum >= 100) && (iDataNum < 104)) ) ||
		!((iDataNum >= 0) && (iDataNum < MAX_DATENSATZ_BESCHREIBUNG)) ||
		(szName == NULL) || (piFieldCount == NULL) /*|| (piVar == NULL)*/ ) {

		theApp.m_log.End("\t\t\t\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {
		
		int iRes;
		UCHAR szTableName[MAX_DS_NAME];
		UCHAR ucFieldCount, ucVariable;

		iRes = pChannel->GetTableRecord(iDataNum, szTableName, &ucFieldCount, &ucVariable, TRUE);
		
		if (iRes) {

			strcpy((char*)szName, (const char*)szTableName);
			*piFieldCount = ucFieldCount;
			// Variable nicht verwendet, kann NULL uebergeben
		}

		theApp.m_log.End("\t%s\t%i\t0\t(%i)", szName, *piFieldCount, iRes);
		return iRes;
	}

	theApp.m_log.End("\t\t\t\t(0)");
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCDatBFeld"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCDatBFeld(int iChannelNum, int iDataNum, int iField, unsigned char *szName, 
						  int *piType, int *piLength)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iDataNum, iField);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!((iDataNum >= 0) && (iDataNum < MAX_DATENSATZ_BESCHREIBUNG)) ||
		!((iField >= 0) && (iField < MAX_FELD)) ||
		(szName == NULL) || (piType == NULL) || (piLength == NULL) ) {

		theApp.m_log.End("\t\t\t\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes;
		UCHAR szFieldName[MAX_DS_NAME];
		UCHAR ucType, ucLength;
		
		iRes = pChannel->GetTableField(iDataNum, iField, szFieldName, 
			&ucType, &ucLength, FALSE);

		if (iRes) {

			strcpy((char*)szName, (const char*)szFieldName);
			*piType = ucType;
			*piLength = ucLength;
		}

		theApp.m_log.End("\t%s\t%i\t%i\t(%i)", szName, *piType, *piLength, iRes);
		return iRes;
	}

	theApp.m_log.End("\t\t\t\t(0)");
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCListBFeld"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCListBFeld(int iChannelNum, int iDataNum, int iField, unsigned char *szName,
						   int *piType, int *piLength)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iDataNum, iField);

	// zulaessige Bereiche pruefen
	if (!( ((iDataNum >= 0) && (iDataNum < MAX_DATENSATZ_BESCHREIBUNG)) ||
		   ((iDataNum >= 100) && (iDataNum < 104)) ) ||
		!((iDataNum >= 0) && (iDataNum < MAX_DATENSATZ_BESCHREIBUNG)) ||
		!((iField >= 0) && (iField < MAX_FELD)) ||
		(szName == NULL) || (piType == NULL) || (piLength == NULL) ) {

		theApp.m_log.End("\t\t\t\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes;
		UCHAR szFieldName[MAX_DS_NAME];
		UCHAR ucType, ucLength;
		
		iRes = pChannel->GetTableField(iDataNum, iField, szFieldName, 
			&ucType, &ucLength, TRUE);

		if (iRes) {

			strcpy((char*)szName, (const char*)szFieldName);
			*piType   = ucType;
			*piLength = ucLength;
		}

		theApp.m_log.End("\t%s\t%i\t%i\t(%i)", szName, *piType, *piLength, iRes);
		return iRes;
	}

	theApp.m_log.End("\t\t\t\t(0)");
	return 0;
}


// #################### +++++++++++++++++ ###########################
// #################### ZUTRITTSKONTROLLE ###########################

#undef FUNC_NAME
#define FUNC_NAME "DFCMakeEntranceList"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCMakeEntranceList(int iChannelNum, int iListNum, int iLineCount, int iListSize, 
						   unsigned char *pucBuf, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iListNum, iLineCount, iListSize);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iListNum >= 0) && (iListNum <= 3)) || (iListNum == -1)) ||
		(pucBuf == NULL) || (piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t\t%i\t(%i)", ERROR_RANGE, 0);
		return 0;
	}
	*piError = ERROR_NO;

	// ListNum ist:
	// 0: Zeitzonen     (22 Byte) zu komprimierten  (6 Byte) max.   16 DS moeglich
	// 1: Zeitmodelle   (35 Byte) zu komprimierten  (3 Byte) max.   32 DS moeglich
	// 2: Personalstamm (19 Byte) zu komprimierten (16 Byte) max. 5120 DS moeglich
	// 3: Feiertage     (16 Byte) zu komprimierten  (6 Byte) max.   64 DS moeglich

	// Bereich der gueltigen Nummer pruefen.
	switch(iListNum) {
	case 0:
		if ((iLineCount < 1) || (iLineCount > 16)) {

			*piError = ERROR_RANGE;
			theApp.m_log.End("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		
		if (iListSize != (iLineCount * ENTRANCE_RECORD_SIZE_0)) {
		
			*piError = ERROR_RANGE;
			theApp.m_log.End("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		break;
	case 1:
		if ((iLineCount < 1) || (iLineCount > 32)) {

			*piError = ERROR_RANGE;
			theApp.m_log.End("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}

		if (iListSize != (iLineCount * ENTRANCE_RECORD_SIZE_1)) {
		
			*piError = ERROR_RANGE;
			theApp.m_log.End("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		break;
	case 2:
		if ((iLineCount < 1) || (iLineCount > 5120)) {

			*piError = ERROR_RANGE;
			theApp.m_log.End("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}

		if (iListSize != (iLineCount * ENTRANCE_RECORD_SIZE_2)) {
		
			*piError = ERROR_RANGE;
			theApp.m_log.End("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		break;
	case 3:
		if ((iLineCount < 1) || (iLineCount > 64)) {

			*piError = ERROR_RANGE;
			theApp.m_log.End("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}

		if (iListSize != (iLineCount * ENTRANCE_RECORD_SIZE_3)) {
		
			*piError = ERROR_RANGE;
			theApp.m_log.End("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		break;

	default:
			*piError = ERROR_RANGE;
			theApp.m_log.End("\t\t%i\t(%i)", *piError, 0);
			return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes = pChannel->MakeEntranceList(iListNum, iLineCount, iListSize, pucBuf);
		theApp.m_log.End("\t\t%i\t(%i)", ERROR_NO, iRes);
		return iRes;

	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t\t%i\t(%i)", *piError, 0);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCClearEntranceListBuffer"
// ########################################################
extern "C" __declspec(dllexport) 

void __stdcall DFCClearEntranceListBuffer(int iChannelNum, int iListNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iListNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iListNum >= 0) && (iListNum <= 3)) || (iListNum == -1)) ) {

		theApp.m_log.End("");
		return;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		pChannel->ClearEntranceListBuffer(iListNum);
	}

	theApp.m_log.End("");
}

#undef FUNC_NAME
#define FUNC_NAME "DFCLoadEntranceList"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCLoadEntranceList(int iChannelNum, int iBusNum, int iListNum, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum, iListNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		!(((iListNum >= 0) && (iListNum <= 3)) || (iListNum == -1)) ||
		(piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t%i\t(%i)", ERROR_RANGE, 0);
		return 0;
	}
	*piError = ERROR_NO;

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes;

		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		pChannel->m_usLastDeviceVersion = 0;
		
		// Pruefen und halten der Firmwareversion
		if (!(iRes = pChannel->CheckComVersion(iBusNum, piError))) {
		
			//iError = ERROR_FW_VERSION;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}
		
		pChannel->m_usLastDeviceVersion = iRes;

		iRes = pChannel->LoadEntranceListData(iBusNum, iListNum, piError);

		if (!iRes && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t%i\t(%i)", *piError, iRes);
		return iRes;

	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t%i\t(%i)", *piError, 0);
	return 0;
}

// ###################### ZUTRITTSKONTROLLE #########################
// ###################### ----------------- #########################

#undef FUNC_NAME
#define FUNC_NAME "DFCSetupLaden"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCSetupLaden(int iChannelNum,int iBusNum, char *szFileName, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i\t%s", FUNC_NAME, iChannelNum, iBusNum, 
		(szFileName == NULL) ? "" : szFileName);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(szFileName == NULL) || (piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t%i\t(%i)", ERROR_RANGE, 0);
		return 0;
	}
	*piError = ERROR_NO;

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes;

		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		// Kommunikationsversion fuer Datenuebertragung
		pChannel->m_usLastDeviceVersion = 0;

		// Pruefen der Firmwareversion
		if (!(iRes = pChannel->CheckComVersion(iBusNum, piError))) {
		
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		// Kommunikationsversion fuer Datenuebertragung setzten
		pChannel->m_usLastDeviceVersion = iRes;

		// Setup-Datei oeffnen
		FILE *pFile = fopen(szFileName, "rb");

		if (pFile == NULL) {
	
			*piError = ERROR_OPEN;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}
		
		// Erweiterter Speicherbereich
		if ((pChannel->m_usLastDeviceVersion == 0x0400) || 
			(pChannel->m_usLastDeviceVersion == 0x0401) ||
			(pChannel->m_usLastDeviceVersion == 0x0402) ||
			(pChannel->m_usLastDeviceVersion == 0x0500) ||
			(pChannel->m_usLastDeviceVersion == 0x0501) ||
			(pChannel->m_usLastDeviceVersion == 0x0502) ) {

			struct stat buf;
			int res = stat(szFileName, &buf);

			// Dateigroesse pruefen
			if ((res != 0) || (buf.st_size < 
				(EEP_SIZE + FLASH_SIZE_SU + 20 + CONFIG_SIZE_SU + EXTENDED_SIZE_SU))) {

				fclose(pFile);

				*piError = ERROR_FORMAT;
				theApp.m_log.End("\t%i\t(%i)", *piError, 0);
				return 0;
			}
		}

		// Speicher initialisieren
		pChannel->ClearMem();

		// EEPROM - lesen
		fread(pChannel->m_pucEepMem, sizeof(unsigned char), EEP_SIZE, pFile);

		// Flashbereich - lesen
		fread(pChannel->m_pucFlashMem, sizeof(unsigned char), FLASH_SIZE_SU, pFile);
		
		// ListFile - lesen
		int i;
		int iLen;
		unsigned char buf[0xFFF];
		for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++) {
		
			fread(buf, sizeof(unsigned char), 1, pFile); // ein byte laenge
			if (buf[0] == 0xFF) {

				fread(buf, sizeof(unsigned char), 2, pFile); // zwei byte laenge
				iLen = *(unsigned short*)buf;

			} else {

				iLen = buf[0];
			}

			fread(buf, sizeof(unsigned char), iLen, pFile);
		}

		// Konfigurationsdaten - lesen
		fread(pChannel->m_pucConfigMem, sizeof(unsigned char), CONFIG_SIZE_SU, pFile);
	
		
		// Erweiterter Flashbereich
		if ((pChannel->m_usLastDeviceVersion == 0x0400) || 
			(pChannel->m_usLastDeviceVersion == 0x0401) ||
			(pChannel->m_usLastDeviceVersion == 0x0402) ||
			(pChannel->m_usLastDeviceVersion == 0x0500) ||
			(pChannel->m_usLastDeviceVersion == 0x0501) ||
			(pChannel->m_usLastDeviceVersion == 0x0502) ) {

			// Erweiterter Flashbereich - lesen
			fread(pChannel->m_pucExtendedMem, sizeof(unsigned char), EXTENDED_SIZE_SU, pFile);

			// Pruefen der Extended - Struktur
			TAE3Extended extBegin;
			for (i=0; i < sizeof(TAE3Extended); i++) {
			
				((UCHAR*)&extBegin)[i] = pChannel->m_pucExtendedMem[i];
			}
			if (extBegin.ulCheck != EXTENDED_CHECK) { // existiert?

				fclose(pFile);

				*piError = ERROR_FORMAT;
				theApp.m_log.End("\t%i\t(%i)", *piError, 0);
				return 0;
			}
		}

		fclose(pFile);

		// Setupdaten uebertragen
		iRes = pChannel->UpLoadSetupData(iBusNum, piError);

		if (!iRes && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();

			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		// Datenpointer loeschen
		iRes = pChannel->ClearData(iBusNum, piError);

		if (!iRes && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t%i\t(%i)", *piError, iRes);
		return iRes;

	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t%i\t(%i)", *piError, 0);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCDownload"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCDownload(int iChannelNum,int iBusNum, char *szFileName, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t%i\t%s", FUNC_NAME, iChannelNum, iBusNum,
		(szFileName == NULL) ? "" : szFileName);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(szFileName == NULL) || (piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t%i\t(%i)", ERROR_RANGE, 0);
		return 0;
	}
	*piError = ERROR_NO;

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {
	
		int iRes;

		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		// Kommunikationsversion fuer Datenuebertragung
		pChannel->m_usLastDeviceVersion = 0;

		// Pruefen der Firmwareversion
		if (!(iRes = pChannel->CheckComVersion(iBusNum, piError))) {
		
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		// Kommunikationsversion fuer Datenuebertragung setzten
		pChannel->m_usLastDeviceVersion = iRes;

		// Setup-Datei oeffnen
		FILE *pFile = fopen(szFileName, "wb");

		if (pFile == NULL) {
	
			*piError = ERROR_OPEN;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		// Speicher initialisieren
		pChannel->ClearMem();

		// Setupdaten aus Geraet holen
		iRes = pChannel->DownLoadSetupData(iBusNum, piError);
		if (!iRes && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();

			fclose(pFile);
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		// EEPROM - schreiben
		fwrite(pChannel->m_pucEepMem, sizeof(unsigned char), EEP_SIZE, pFile);

		// Flashbereich - schreiben
		fwrite(pChannel->m_pucFlashMem, sizeof(unsigned char), FLASH_SIZE_SU, pFile);
		
		// Leere Filelist schreiben
		unsigned char buf[MAX_DATENSATZ_BESCHREIBUNG];
		memset(buf, 0, MAX_DATENSATZ_BESCHREIBUNG);

		fwrite(buf, sizeof(unsigned char), MAX_DATENSATZ_BESCHREIBUNG, pFile);

		// Konfiguration - schreiben
		fwrite(pChannel->m_pucConfigMem, sizeof(unsigned char), CONFIG_SIZE_SU, pFile);

		// Erweiterter Flashbereich
		if ((pChannel->m_usLastDeviceVersion == 0x0400) || 
			(pChannel->m_usLastDeviceVersion == 0x0401) ||
			(pChannel->m_usLastDeviceVersion == 0x0402) ||
			(pChannel->m_usLastDeviceVersion == 0x0500) ||
			(pChannel->m_usLastDeviceVersion == 0x0501) ||
			(pChannel->m_usLastDeviceVersion == 0x0502) ) {
			
			// Erweiterter Flashbereich - schreiben
			fwrite(pChannel->m_pucExtendedMem, sizeof(unsigned char), EXTENDED_SIZE_SU, pFile);
		}

		// Datei schlieszen
		fclose(pFile);

		// Setupdaten in Datei geschrieben
		theApp.m_log.End("\t%i\t(%i)", *piError, 1);
		return 1;
	}

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t%i\t(%i)", *piError, 0);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComVersion"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCComVersion(int iChannelNum, int iBusNum, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t%i\t(%i)", ERROR_RANGE, 0);
		return 0;
	}
	*piError = ERROR_NO;

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes;

		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		iRes = pChannel->CheckComVersion(iBusNum, piError);

		if (!iRes && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t%i\t(0x%04X)", *piError, iRes);
		// Kommunikationsversion z.B: 0x00000103
		// Die Versionsnummer ist ein 16Bit - Wert
		return iRes;
	
	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t%i\t(%i)", *piError, 0);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetVersionFirmware"
// ########################################################
extern "C" __declspec(dllexport)
 
int __stdcall DFCGetVersionFirmware(int iChannelNum, int iBusNum, char *szVersion, int *piError)
{
	AFX_MANAGE_STATE(AfxGetAfxStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(szVersion == NULL) || (piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t\t%i\t(%i)", ERROR_RANGE, 0);
		return 0;
	}
	*piError = ERROR_NO;

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes;
		
		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t\t%i\t(%i)", *piError, 0);
			return 0;
		}

		pChannel->m_usLastDeviceVersion = 0;
		
		// Pruefen und halten der Firmwareversion
		if (!(iRes = pChannel->CheckComVersion(iBusNum, piError))) {
		
			//*piError = ERROR_FW_VERSION;
			theApp.m_log.End("\t\t%i\t(0)", *piError, 0);
			return 0;
		}
		
		pChannel->m_usLastDeviceVersion = iRes;

		iRes = pChannel->GetVersionFirmware(iBusNum, szVersion, piError);

		if (!iRes && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t%s\t%i\t(%i)", (iRes) ? szVersion : "", *piError, iRes);
		return iRes;
	}

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t\t%i\t(0)", *piError);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetVersionFirmwareFromFile"
// ########################################################
extern "C" __declspec(dllexport)
 
int __stdcall DFCGetVersionFirmwareFromFile(int iChannelNum, char *szFileName, char *szVersion, int *piError)
{
	AFX_MANAGE_STATE(AfxGetAfxStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i", FUNC_NAME, iChannelNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		(szFileName == NULL) || (szVersion == NULL) || (piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t\t\t%i\t(%i)", ERROR_RANGE, 0);
		return 0;
	}
	*piError = ERROR_NO;

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes;
		
		iRes = pChannel->GetVersionFirmwareFromFile(szFileName, szVersion, piError);

		if (!iRes && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t%s\t%s\t%i\t(%i)", szFileName, (iRes) ? szVersion : "", *piError, iRes);
		return iRes;
	}

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t%s\t\t%i\t(0)", szFileName, *piError);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetSeriennummer"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCGetSeriennummer(int iChannelNum, int iBusNum, int *piError, int *piSerialNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) || (piSerialNum == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t%i\t\t(%i)", ERROR_RANGE, 0);
		return 0;
	}
	*piError = ERROR_NO;

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channel vorhanden
	if (pChannel != NULL) {

		int iRes;

		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t%i\t\t(%i)", *piError, 0);
			return 0;
		}

		iRes = pChannel->GetSerialNum(iBusNum, piSerialNum, piError);

		if (!iRes && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t%i\t%i\t(%i)", *piError, *piSerialNum, iRes);
		return iRes;

	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t%i\t\t(%i)", *piError, 0);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSetCallBack"
// ########################################################
extern "C" __declspec(dllexport) 

void __stdcall DFCSetCallBack(int iChannelNum, int (*CBFunction)(void))
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t", FUNC_NAME, iChannelNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ) {

		theApp.m_log.End("");
		return;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channel vorhanden
	if (pChannel != NULL) {

		pChannel->SetCBFunction(CBFunction);
	} 

	theApp.m_log.End("");
}

#undef FUNC_NAME
#define FUNC_NAME "DFCCheckData"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCCheckData(int iChannelNum, int iBusNum, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t%i\t(%i)", ERROR_RANGE, -1);
		return -1;
	}
	*piError = ERROR_NO;

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	if (pChannel != NULL) {

		int iRes;
		
		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t%i\t(%i)", *piError, -1);
			return -1;
		}

		pChannel->m_usLastDeviceVersion = 0;
		
		// Pruefen und halten der Firmwareversion
		if (!(iRes = pChannel->CheckComVersion(iBusNum, piError))) {
		
			//iError = ERROR_FW_VERSION;
			theApp.m_log.End("\t%i\t(%i)", *piError, -1);
			return -1;
		}
		
		pChannel->m_usLastDeviceVersion = iRes;

		iRes = pChannel->CheckData(iBusNum, piError);

		if ((iRes < 0) && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t%i\t(%i)", *piError, iRes);
		return iRes;
	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t%i\t(%i)", *piError, -1);
	return -1;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCSetGlobVar"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCSetGlobVar(int iChannelNum, int iBusNum, unsigned char *pucVar, int iType, unsigned char *pucValue, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(pucVar == NULL) || (pucValue == NULL) || (piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t\t%i\t\t%i\t(%i)", iType, ERROR_RANGE, 0);
		return 0;
	}
	*piError = ERROR_NO;

	// Bereich pruefen.
	switch(iType) {
	case 0: // Ansprechen ueber Nummer
		if (*pucVar < 1 || *pucVar > 8) {

			*piError = ERROR_RANGE;
			theApp.m_log.End("\t\t%i\t\t%i\t(%i)", iType, *piError, 0);
			return 0;
		}
		theApp.m_log.Append("\t%i\t%i\t%s", *pucVar, iType, (char*)pucValue);
		break;
	case 1: // Ansprechen ueber Name
		if ((strlen((const char*)pucVar) < 1) || (strlen((const char*)pucVar) > MAX_S_FELD_TXT)) {

			*piError = ERROR_RANGE;
			theApp.m_log.End("\t\t%i\t\t%i\t(%i)", iType, *piError, 0);
			return 0;
		}
		theApp.m_log.Append("\t%s\t%i\t%s", (char*)pucVar, iType, (char*)pucValue);
		break;
	case 2: // Ansprechen ueber Systemname
		if (strlen((const char*)pucVar) < 1) {

			*piError = ERROR_RANGE;
			theApp.m_log.End("\t\t%i\t\t%i\t(%i)", iType, *piError, 0);
			return 0;
		}
		theApp.m_log.Append("\t%s\t%i\t%s", (char*)pucVar, iType, (char*)pucValue);
		break;
	default:
		*piError = ERROR_RANGE;
		theApp.m_log.End("\t\t%i\t\t%i\t(%i)", iType, *piError, 0);
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	if (pChannel != NULL) {

		int iRes;
		
		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		pChannel->m_usLastDeviceVersion = 0;
		
		// Pruefen und halten der Firmwareversion
		if (!(iRes = pChannel->CheckComVersion(iBusNum, piError))) {
		
			//iError = ERROR_FW_VERSION;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}
		
		pChannel->m_usLastDeviceVersion = iRes;

		iRes = pChannel->SetGlobVar(iBusNum, pucVar, iType, pucValue, piError);

		if (!iRes && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t%i\t(%i)", *piError, iRes);
		return iRes;

	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t%i\t(%i)", *piError, 0);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetGlobVar"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCGetGlobVar(int iChannelNum, int iBusNum, unsigned char *pucVar, int iType, unsigned char *pucValue, int iValueLength, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(pucVar == NULL) || (pucValue == NULL) || (piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t\t%i\t\t%i\t%i\t(%i)", iType, iValueLength, ERROR_RANGE, 0);
		return 0;
	}
	*piError = ERROR_NO;

	// Bereich pruefen.
	switch(iType) {
	case 0: // Ansprechen ueber Nummer
		if (*pucVar < 1 || *pucVar > 8) {

			*piError = ERROR_RANGE;
			theApp.m_log.End("\t\t%i\t\t%i\t%i\t(%i)", iType, iValueLength, *piError, 0);
			return 0;
		}
		theApp.m_log.Append("\t%i\t%i", *pucVar, iType);
		break;
	case 1: // Ansprechen ueber Name
		if ((strlen((const char*)pucVar) < 1) || (strlen((const char*)pucVar) > MAX_S_FELD_TXT)) {

			*piError = ERROR_RANGE;
			theApp.m_log.End("\t\t%i\t\t%i\t%i\t(%i)", iType, iValueLength, *piError, 0);
			return 0;
		}
		theApp.m_log.Append("\t%s\t%i", (char*)pucVar, iType);
		break;
	case 2: // Ansprechen ueber Systemname
		if (strlen((const char*)pucVar) < 1) {

			*piError = ERROR_RANGE;
			theApp.m_log.End("\t\t%i\t\t%i\t%i\t(%i)", iType, iValueLength, *piError, 0);
			return 0;
		}
		theApp.m_log.Append("\t%s\t%i", (char*)pucVar, iType);
		break;
	default:
		*piError = ERROR_RANGE;
		theApp.m_log.End("\t\t%i\t\t%i\t%i\t(%i)", iType, iValueLength, *piError, 0);
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	if (pChannel != NULL) {

		int iRes;
		
		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t\t%i\t%i\t(%i)", iValueLength, *piError, 0);
			return 0;
		}

		pChannel->m_usLastDeviceVersion = 0;
		
		// Pruefen und halten der Firmwareversion
		if (!(iRes = pChannel->CheckComVersion(iBusNum, piError))) {
		
			//iError = ERROR_FW_VERSION;
			theApp.m_log.End("\t\t%i\t%i\t(%i)", iValueLength, *piError, 0);
			return 0;
		}
		
		pChannel->m_usLastDeviceVersion = iRes;

		iRes = pChannel->GetGlobVar(iBusNum, pucVar, iType, pucValue, iValueLength, piError);

		if (!iRes && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}
		
		theApp.m_log.End("\t%s\t%i\t%i\t(%i)", (char*)pucValue, iValueLength, *piError, iRes);
		return iRes;

	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t\t%i\t%i\t(%i)", iValueLength, *piError, 0);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCCloseRelay"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCCloseRelay(int iChannelNum, int iBusNum, int iNum, int iTime, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum, iNum, iTime);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t%i\t(%i)", ERROR_RANGE, 0);
		return 0;
	}
	*piError = ERROR_NO;

	// Bereich der gueltigen Nummer pruefen.
	if ((iNum < 0) || (iNum > 8)) {
	
		*piError = ERROR_RANGE;
		theApp.m_log.End("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes;
		
		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		pChannel->m_usLastDeviceVersion = 0;
		
		// Pruefen und halten der Firmwareversion
		if (!(iRes = pChannel->CheckComVersion(iBusNum, piError))) {
		
			//iError = ERROR_FW_VERSION;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}
		
		pChannel->m_usLastDeviceVersion = iRes;

		iRes = pChannel->CloseRelay(iBusNum, iNum, iTime, piError);

		if (!iRes && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t%i\t(%i)", *piError, iRes);
		return iRes;

	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t%i\t(%i)", *piError, 0);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCGetRelayState"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCGetRelayState(int iChannelNum, int iBusNum, int iNum, int *piState, int *piTime, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum, iNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piState == NULL) || (piTime == NULL) || (piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t\t\t%i\t(%i)", ERROR_RANGE, 0);
		return 0;
	}
	*piError = ERROR_NO;

	// Bereich der gueltigen Nummer pruefen.
	if ((iNum < 0) || (iNum > 8)) {
	
		*piError = ERROR_RANGE;
		theApp.m_log.End("\t\t\t%i\t(%i)", *piError, 0);
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes;
		
		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t\t\t%i\t(%i)", *piError, 0);
			return 0;
		}

		pChannel->m_usLastDeviceVersion = 0;
		
		// Pruefen und halten der Firmwareversion
		if (!(iRes = pChannel->CheckComVersion(iBusNum, piError))) {
		
			//iError = ERROR_FW_VERSION;
			theApp.m_log.End("\t\t\t%i\t(%i)", *piError, 0);
			return 0;
		}
		
		pChannel->m_usLastDeviceVersion = iRes;

		iRes = pChannel->GetRelayState(iBusNum, iNum, piState, piTime, piError);

		if (!iRes && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t%i\t%i\t%i\t(%i)", *piState, *piTime, *piError, 0);
		return iRes;

	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t\t\t%i\t(%i)", *piError, 0);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCOpenRelay"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCOpenRelay(int iChannelNum, int iBusNum, int iNum, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum, iNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t%i\t(%i)", ERROR_RANGE, 0);
		return 0;
	}
	*piError = ERROR_NO;

	// Bereich der gueltigen Nummer pruefen.
	if ((iNum < 0) || (iNum > 8)) {
	
		*piError = ERROR_RANGE;
		theApp.m_log.End("\t%i\t(%i)", *piError, 0);
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes;
		
		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		pChannel->m_usLastDeviceVersion = 0;
		
		// Pruefen und halten der Firmwareversion
		if (!(iRes = pChannel->CheckComVersion(iBusNum, piError))) {
		
			//iError = ERROR_FW_VERSION;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}
		
		pChannel->m_usLastDeviceVersion = iRes;

		iRes = pChannel->OpenRelay(iBusNum, iNum, piError);

		if (!iRes && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t%i\t(%i)", *piError, iRes);
		return iRes;

	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t%i\t(%i)", *piError, 0);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCWrite"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCWrite(int iChannelNum, char *pBuf, int iLength, int *piWriteLength, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i", FUNC_NAME, iChannelNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		(pBuf == NULL) || (piWriteLength == NULL) ||
		(piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t\t%i\t\t%i\t(0)", iLength, ERROR_RANGE);
		return 0;
	}
	*piError = ERROR_NO;

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	if (pChannel != NULL) {

		int iRes;

		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t%i\t\t%i\t(0)", iLength, *piError);
			return 0;
		}

		iRes = pChannel->Write(pBuf, iLength, (unsigned long*)piWriteLength);
		theApp.m_log.End("\t%s\t%i\t%i\t%i\t(%i)", (strlen(pBuf) <= (unsigned int)iLength) ? pBuf : "", 
			iLength, *piWriteLength, *piError, iRes);
		
		return iRes;
	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t\t%i\t\t%i\t(0)", iLength, *piError);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCRead"
// ########################################################
extern "C" __declspec(dllexport)

int __stdcall DFCRead(int iChannelNum, char *pBuf, int iLength, int *piReadLength, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	theApp.m_log.Begin("%s\t%i", FUNC_NAME, iChannelNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		(pBuf == NULL) || (piReadLength == NULL) ||
		(piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t\t%i\t\t%i\t(0)", iLength, ERROR_RANGE);
		return 0;
	}
	*piError = ERROR_NO;

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	if (pChannel != NULL) {

		int iRes;

		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t%i\t\t%i\t(0)", iLength, *piError);
			return 0;
		}

		iRes = pChannel->Read(pBuf, iLength, (unsigned long*)piReadLength);
		theApp.m_log.End("\t%s\t%i\t%i\t%i\t(%i)", (strlen(pBuf) <= (unsigned int)iLength) ? pBuf : "", 
			iLength, *piReadLength, *piError, iRes);
		
		return iRes;
	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t\t%i\t\t%i\t(0)", iLength, *piError);
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCUpload"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCUpload(int iChannelNum, int iBusNum, char *szFileName, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i\t%i\t%s", FUNC_NAME, iChannelNum, iBusNum, 
		(szFileName == NULL) ? "" : szFileName);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(szFileName == NULL) || (piError == NULL) ) {

		if (piError) *piError = ERROR_RANGE;
		theApp.m_log.End("\t%i\t(%i)", ERROR_RANGE, 0);
		return 0;
	}
	*piError = ERROR_NO;

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes;

		// Schnittstelle geoeffnet und CCom-Kanalklasse?
		if (!pChannel->IsOpen() || (pChannel->GetChannelType() != CChannel::Class)) {

			*piError = ERROR_COM_CLOSED;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		// Zukuenftig evntl. anhand der Endung entscheiden!
		// Nur ".hex" gueltig.
		if (strcmp(&szFileName[strlen(szFileName) - 4], ".hex") != 0) {

			*piError = ERROR_OPEN;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}

		// Dateidaten laden
		pChannel->m_fileDevice.Reset();
		if (!pChannel->m_fileDevice.Load(szFileName) || pChannel->m_fileDevice.IsCrypt()) {

			pChannel->m_fileDevice.Reset();
	
			*piError = ERROR_OPEN;
			theApp.m_log.End("\t%i\t(%i)", *piError, 0);
			return 0;
		}
		
		// Daten uebertragen
		iRes = pChannel->UploadFileDevice(iBusNum, piError);

		if (!iRes && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t%i\t(%i)", *piError, iRes);
		return iRes;

	} 

	*piError = ERROR_CHANNEL;
	theApp.m_log.End("\t%i\t(%i)", *piError, 0);
	return 0;
}



// ########################################################
// #############    ALT   #################################
// ########################################################

#undef FUNC_NAME
#define FUNC_NAME "DFCLogOn"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCLogOn(char *szFileName)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%s", FUNC_NAME, (szFileName == 0) ? "" : szFileName);
	
	theApp.m_log.End("\t(%i)", 0);
	// Wird nicht mehr unterstuetzt!
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCLogOff"
// ########################################################
extern "C" __declspec(dllexport)
 
void __stdcall DFCLogOff(void)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s", FUNC_NAME);
	theApp.m_log.End("");
	// Wird nicht mehr unterstuetzt!
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComUseHandle"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCComUseHandle(int iChannelNum, HANDLE hCom)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, hCom);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ) {

		theApp.m_log.End("\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes = pChannel->ComUseHandle(hCom);
		theApp.m_log.End("\t(%i)", iRes);
		return iRes;
	}

	theApp.m_log.End("\t(0)");
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComReleaseHandle"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCComReleaseHandle(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i", FUNC_NAME, iChannelNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ) {

		theApp.m_log.End("\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes = pChannel->ComReleaseHandle();
		theApp.m_log.End("\t(%i)", iRes);
		return iRes;
	}

	theApp.m_log.End("\t(0)");
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComInit"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCComInit(int iChannelNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	theApp.m_log.Begin("%s\t%i", FUNC_NAME, iChannelNum);

	// zulaessige Bereiche pruefen
	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ) {

		theApp.m_log.End("\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {
	
		char szComStr[14];

#ifdef _WIN32
		if (iChannelNum > 9) {
			sprintf(szComStr, "\\\\?\\COM%i", iChannelNum);
		} else {
			sprintf(szComStr, "COM%i", iChannelNum);
		}

#else
		sprintf(szComStr, "/dev/ttyS%i", iChannelNum - 1);
#endif

		int iRes = pChannel->OpenSerial(szComStr, 38400, 150);
		theApp.m_log.End("\t(%i)", iRes);
		return iRes;
	}

	theApp.m_log.End("\t(0)");
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCComOpen"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCComOpen(int iChannelNum, char *szHost)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t%s", FUNC_NAME, iChannelNum, (szHost == NULL) ? "" : szHost);

	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		(szHost == NULL) ) {

		theApp.m_log.End("\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channel vorhanden
	if (pChannel != NULL) {

		int iRes = pChannel->OpenSocket(szHost, 8000, 3000);
		theApp.m_log.End("\t(%i)", iRes);
		return iRes;
	}

	theApp.m_log.End("\t(0)");
	return 0;
}

#undef FUNC_NAME
#define FUNC_NAME "DFCCheckAE"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCCheckAE(int iChannelNum, int iBusNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ) {

		theApp.m_log.End("\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iError = ERROR_NO;
		int iRes;

		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			theApp.m_log.End("\t(0)");
			return 0;
		}

		iRes = pChannel->CheckDevice(iBusNum, &iError, 1);

		if (!iRes && (iError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t(%i)", iRes);
		return iRes;
	}

	theApp.m_log.End("\t(0)");
	return 0;
}


#undef FUNC_NAME
#define FUNC_NAME "DFCWatchdogReset"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCWatchdogReset(int iChannelNum, int iBusNum)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ) {

		theApp.m_log.End("\t(0)");
		return 0;
	}

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iError = ERROR_NO;
		int iRes;

		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			theApp.m_log.End("\t(0)");
			return 0;
		}

		iRes = pChannel->Restart(iBusNum, &iError);

		if (!iRes && (iError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t(%i)", iRes);
		return iRes;
	}

	theApp.m_log.End("\t(0)");
	return 0;
}


/*
#undef FUNC_NAME
#define FUNC_NAME "DFCCheckCmd"
// ########################################################
extern "C" __declspec(dllexport) 

int __stdcall DFCCheckCmd(int iChannelNum, int iBusNum, int *piError)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	theApp.m_log.Begin("%s\t%i\t%i", FUNC_NAME, iChannelNum, iBusNum);

	if (!((iChannelNum >= 0) && (iChannelNum < MAX_CHANNEL)) ||
		!(((iBusNum >= 1) && (iBusNum <= 31)) || (iBusNum == 254)) ||
		(piError == NULL)) {

		theApp.m_log.End("\t(0)");
		return 0;
	}
	*piError = ERROR_NO;

	CAE3Channel *pChannel   = theApp.GetChannel(iChannelNum);

	// Pruefe ob Channelobjekt vorhanden
	if (pChannel != NULL) {

		int iRes;

		// Schnittstelle geoeffnet?
		if (!pChannel->IsOpen()) {

			theApp.m_log.End("\t(0)");
			return 0;
		}

		iRes = pChannel->CheckCmd(iBusNum, piError);

		if (!iRes && (*piError != ERROR_NO)) {
		
			pChannel->ClearCom();
		}

		theApp.m_log.End("\t(%i)", iRes);
		return iRes;
	}

	theApp.m_log.End("\t(0)");
	return 0;
}
*/
